// NOTE: this directive send event to google analytics
// Accept differents params separated for commas (,):

// If for a **event** default accept:
// - type of action: hover, focus, click(default) (required)  | ga=""
// - eventCategory: string (required)                         | ga-cat=""
// - Action: string (required)                                | ga-action=""
// - Label: string (optional)                                 | ga-label=""
// - Value: number Int (optional)                             | ga-val=""

// If for a **event** default accept:
// - type of action: hover, focus, click(default) (required)  | ga=""
// - hitType: social (required)                               | ga-type=""
// - socialNetwork: facebook, twitter, linkedin,...(required) | ga-network=""
// - socialAction: like, follow, tweet,... (required)         | ga-action=""
// - socialTarget: url shared (optional)                      | ga-target=""

//EX: <input ng-model="jobberLead.email"
//      ga="focus" ga-type="event"
//      ga-cat="professionals-page" ga-label="focus-email" ga-val="2"
//      ga-action="put-email">
//    </input>
// https://developers.google.com/analytics/devguides/collection/analyticsjs/events?hl=es

angular.module('jobmapp.generics.directives')
.directive("ga", [
  '$window',
  '$rootScope',
  function ($window, $rootScope) {
    return function(scope, element, attrs) {
      // event trigger, default click
      if (attrs.ga == 'hover') {
        element.bind('hover', function() {
          choosehit();
        })
      }
      else if (attrs.ga == 'focus') {
        element.bind('focus', function() {
          choosehit();
        })
      }
      else{
        element.bind('click', function() {
          choosehit();
        })
      }

      function choosehit() {
        if(attrs.gaType == 'social'){
          sendSocialEvent();
        }
        else {
          sendEvent();
        }
      }

      function sendSocialEvent() {
        var obj = {}
        obj.hitType = 'social'

        if(attrs.gaNetwork)
        obj.socialNetwork = attrs.Network

        if(attrs.gaAction)
        obj.socialAction   = attrs.gaAction

        if(attrs.gaTarget)
        obj.socialTarget = attrs.gaTarget

        if (ga) {
          ga('send', obj);
        }
      }

      function sendEvent() {
        var obj  = {};
        var mixp = {
          action:   null,
          options:  {}
        };

        if(attrs.gaType) {
          obj.hitType = attrs.gaType
        } else {
          obj.hitType = 'event'
        }

        if(attrs.gaAction)
        obj.eventAction   = attrs.gaAction;
        mixp.action       = attrs.gaAction;

        if(attrs.gaCat) {
          obj.eventCategory         = attrs.gaCat;
          mixp.options['category']  = attrs.gaCat;
        }

        if(attrs.gaLabel) {
          obj.eventLabel    = attrs.gaLabel;
          var labels = attrs.gaLabel.split(', ');
          angular.forEach(labels, function(label) {
            var k = label.split(':')[0]
            var v = label.split(':')[1]
            mixp.options[k] = v
          })
        }

        if(attrs.gaVal) {
          obj.eventValue        = attrs.gaVal;
          mixp.options['value'] = attrs.gaVal;
        }

        if (ga) {
          ga('send', obj);
        }

        if (mixpanel) {
          var trackEvent = true;
          if ($rootScope.currentUser) {
            var jobmappTeam = {2:true, 6:true, 54:true, 90:true, 3031:true};
            if (jobmappTeam[$rootScope.currentUser.id]) {
              var trackEvent = false;
            }
          }
          if (trackEvent) {
            mixpanel.track(mixp.action, mixp.options);
          }
        }
      }
    };
  }
]);
