angular.module('jobmapp.generics.resources')
.factory('Job', [
  '$resource',
  'jobCategories',
  'GeoLocation',
  '$q',
  'env',
  'MessageService',
  'ServiceExtentions',
  function($resource, jobCategories, GeoLocation, $q, env, MessageService, ServiceExtentions){
    // get jobCategory.name and jobSubcategory.name from jobCategories and add it to objects:
    var transformJobRequest = function(obj, headers) {
      // var job = obj.job;
      // Keep the original Date object
      var parkedDate = obj.job.date;
      var parkedTime = obj.job.time;
      // Transform date attribute to a String... so that Rails likes it.

      if (obj.job.time && (typeof(obj.job.time) != 'undefined' && obj.job.time != null)) {
        obj.job.date = moment(obj.job.date).hours(moment(obj.job.time).hours()).minutes(moment(obj.job.time).minutes()).seconds(moment(obj.job.time).seconds());
        obj.job.time = null;
      } else {
        obj.job.date = moment(moment(obj.job.date).format('YYYY-MM-DD') + " 23:59:59");
        obj.job.time = null;
      }

      var jsonString = angular.toJson(obj);
      // Set date attribute back to the old Date object.
      obj.job.date = parkedDate;
      obj.job.time = parkedTime;
      return jsonString;
    }

    var transformJobResponse = function(job) {
      job.jobCategory = jobCategories.get(job.category_id).name;
      // OLD ONE: if (job.sub_category_id) {job.jobSubcategory = jobCategories.get(job.sub_category_id).name};
      // TBD: replace standard thumnail with jobcategory icon:
      job.thumbPath = "img/categories/icons_assertive/" + jobCategories.get(job.category_id).icon;
      job.thumbPathWhite = "img/categories/icons_white/" + jobCategories.get(job.category_id).icon;

      job.imagePath = "img/categories/icons_assertive/" + jobCategories.get(job.category_id).icon;
      job.imagePathWhite = "img/categories/icons_white/" + jobCategories.get(job.category_id).icon;

      job.photoServerPathByName = {};
      job.photoThumbServerPathByName = {};

      if (job.photo_file_names.length > 0) {
        job.thumbPath = env.apiEndPoint + "/uploads/jobs/final/thumb_" + job.photo_file_names[0];
        job.thumbPathWhite = env.apiEndPoint + "/uploads/jobs/final/thumb_" + job.photo_file_names[0];
        job.imagePath = env.apiEndPoint + "/uploads/jobs/final/" + job.photo_file_names[0];
        job.imagePathWhite = env.apiEndPoint + "/uploads/jobs/final/" + job.photo_file_names[0];
        job.photoPaths = _.map(job.photo_file_names, function(fileName){
          var path = env.apiEndPoint + "/uploads/jobs/final/" + fileName;
          var thumbPath = env.apiEndPoint + "/uploads/jobs/final/thumb_" + fileName;
          job.photoServerPathByName[fileName] = path;
          job.photoThumbServerPathByName[fileName] = thumbPath;
          return path;
        })
      } else {
        job.photoPaths = [];
        job.photoPaths[0] = "img/categories/icons_white/" + jobCategories.get(job.category_id).icon;
      }
      if (GeoLocation.currentLocation) {
        if (job.position) {
          job.distance = GeoLocation.distanceToCurrent(job.position)
        };
      } else {
        job.distance = '';
      }

      if (job.messages && job.messages.length > 0) {
        _.map(job.messages, function(message){
          return MessageService.transformResponse(message);
        })
      };

      // check if at least one private msg for this job and set privateChat
      if (job.i_created || job.i_am_assigned || (_.findIndex(job.messages, {'private': true}) >= 0 && job.status == 'open')) job.privateChatAllowed = true;
      else job.privateChatAllowed = false;

      if (job.candidates && job.candidates.length > 0 && job.i_created && job.status != 'open') job.privateChatLimitedTo = job.candidates[0].id;

      // Input type="number" fields need to be populated with floats. So make sure, that the prices are floats.
      job.price = parseFloat(job.price);

      jobdate = new Date(job.date);
      // use momentjs.calendar with adopted es.js to show yesterday/tomorrow
      // and if time set to 23:59:59 remove execution time and only show date
      job.dateTimeString = moment(jobdate).calendar();
      job.createdAtString = moment(job.created_at).fromNow(true);

      //If date of job is between yestarday and tomorrow creat day (used in job page on card price)
      var easyDayEn  =  moment(jobdate).locale('en').calendar().split(' ')[0];
      if ( easyDayEn !=  'Today' && easyDayEn !=  'Tomorrow' && easyDayEn !=  'Yesterday' ) {
        job.easyDay = moment(job.date).format('dddd');
      } else {
        job.easyDay  =  moment(jobdate).calendar().split(' ')[0];
      }

      // TEMP WORKAROUND TO BE REMOVED:
      // mark as done if assigned and NOW is after jobdate (counting days, so same day will not be done)
      if (job.status == "assigned" && moment().isAfter(jobdate, 'day')) {
        job.status = "done";
      }
      // candiate thumnail
      if (job.candidates) {
        job.candidates_by_id = {};
        job.candidates = _.map(job.candidates, function (candidate) {
          job.candidates_by_id[candidate.id] = candidate;
          if (candidate.photo_file_names && candidate.photo_file_names.length > 0) {
            candidate.thumbPath = env.apiEndPoint + "/uploads/users/thumb_" + _.last(candidate.photo_file_names);
            candidate.photoPaths = _.map(candidate.photo_file_names, function(fileName){
              return env.apiEndPoint + "/uploads/users/" + fileName;
            })
          } else {
            candidate.thumbPath = "img/user_dark.png";
            candidate.photoPaths = ["img/user_dark.png"];
          }
          return candidate;
        });

      }

      if (job.service) {
        job.service = ServiceExtentions.transformResponse(job.service);
      }

      return job;
    };

    return $resource(env.apiEndPoint + '/jobs/:id.json' , {id: '@id'}, {
      apply: {
        method:'PUT',
        url: env.apiEndPoint + '/jobs/:id/apply.json',
        params: {id: '@job.id'}
      },
      cancelApplication: {
        method:'PUT',
        url: env.apiEndPoint + '/jobs/:id/cancel-application.json',
        params: {id: '@id'}
      },
      cancelAssignment: {
        method:'PUT',
        url: env.apiEndPoint + '/jobs/:id/cancel-assignment.json',
        params: {id: '@id'}
      },
      cancelJob: {
        method:'PUT',
        url: env.apiEndPoint + '/jobs/:id/cancel.json',
        params: {id: '@id'}
      },
      activate: {
        method:'PUT',
        url: env.apiEndPoint + '/jobs/:id/activate.json',
        params: {id: '@id'}
      },
      deny: {
        method:'PUT',
        url: env.apiEndPoint + '/jobs/:id/deny.json',
        params: {id: '@id'}
      },
      assign: {
        method:'PUT',
        url: env.apiEndPoint + '/jobs/:id/assign/:candidateId.json',
        params: {id: '@id', candidateId: '@candidateId'}
      },
      rateJobber: {
        method:'PUT',
        url: env.apiEndPoint + '/jobs/:id/rate-jobber.json',
        params: {id: '@id'}
      },
      rateCreator: {
        method:'PUT',
        url: env.apiEndPoint + '/jobs/:id/rate-creator.json',
        params: {id: '@id'}
      },
      save: {
        method:'POST',
        transformRequest: transformJobRequest,
        params: {id: '@job.id'}
      },
      update: {
        method:'PUT',
        transformRequest: transformJobRequest,
        params: {id: '@job.id'}
      },
      query:  {
        method:'GET',
        transformResponse: function (data) {
          data = JSON.parse(data);
          if (data.length) {
            data = _.map(data, transformJobResponse);
          }
          return data;
        },
        isArray:true
      },
      myJobsCreated: {
        method:'GET',
        url: env.apiEndPoint + '/created/:id.json', // to be changed to "created"
        params: {id: '@id'},
        transformResponse: function (data) {
          data = JSON.parse(data);
          if (data.length) {
            data = _.map(data, transformJobResponse);
          }
          return data;
        },
        isArray:true
      },
      myJobsApplied:  {
        method:'GET',
        url: env.apiEndPoint + '/applied/:id.json',
        params: {id: '@id'},
        transformResponse: function (data) {
          data = JSON.parse(data);
          if (data.length) {
            data = _.map(data, transformJobResponse);
          }
          return data;
        },
        isArray:true
      },
      calculatePrice: {
        method: 'POST',
        url: env.apiEndPoint + '/jobs/calculate-price.json',
      },
      get: {
        method: 'GET',
        transformResponse: function (data) {
          data = JSON.parse(data);
          return transformJobResponse(data);
        },
        isArray: false
      },
    })
  }
]);
