angular.module('jobmapp.generics.services')
.factory('AuthService', [
  '$http',
  'Session',
  'User',
  'env',
  '$q',
  '$timeout',
  // 'BranchIo',
  function($http, Session, User, env, $q, $timeout) {
    var authService = {};

    var saveUserType = function(user) {
      console.log("saveUserType", user);
      var changed = false;
      var is_jobber = (window.localStorage.getItem("is_jobber") == "true");
      var is_creator = (window.localStorage.getItem("is_creator") == "true");
      var language = window.localStorage.getItem("language");
      var updateData = {};
      if (user.language != language) {
        updateData.language = language;
        changed = true;
      }
      if (user.is_jobber != true && is_jobber == true) {
        console.log("is_jobber changed");
        updateData.is_jobber = true;
        changed = true;
      }
      if (user.is_creator != true && is_creator == true) {
        console.log("is_creator changed");
        updateData.is_creator = true;
        changed = true;
      }
      if (changed) {
        User.update(updateData, function(returndata) {
          // window.localStorage.removeItem("is_jobber");
          // window.localStorage.removeItem("is_creator");
        });
      }
    }

    authService.login = function (credentials) {
      // Send data to backend for verification and log in.
      console.log("authService.login", credentials);
      return User.login({user: credentials})
        .$promise.then(
          function (res) {
            if (mixpanel) {
              // Identify with alias the new user
              mixpanel.alias(res.id);
              // Set or update mixpanel user
              mixpanel.people.set({
                "UUID":     res.uuid,
                "$name":    res.name,
                "$created": res.created_at,
                "$email":   res.email
              });
            };

            Session.create(res.authentication_token, res.uuid, res.role);
            // persistant storage of uuid and auth_token:
            window.localStorage.setItem("uuid", res.uuid);
            window.localStorage.setItem("token",res.authentication_token);
            window.localStorage.setItem("role",res.role);
            saveUserType(res);
            return res;
          },
          function (error) {
            console.log("error login", error); // error message sent by user.js
            return $q.reject(error);
          }
        );
    };

    authService.fbLogin = function (fbToken, fbUid, email, phone) {
      console.log("authService.fbLogin", fbToken, fbUid, email, phone);
      // Send data to backend for verification and log in.
      console.log("fbLogin in auth service");
      return User.fbLogin({token: fbToken, uid: fbUid})
        .$promise.then(function (res) {
          if (mixpanel) {
            if (res.just_created_via_fb) {
              mixpanel.identify(res.id);
            } else {
              // Identify with alias the new user
              mixpanel.alias(res.id);
            }
            // Set or update mixpanel user
            mixpanel.people.set({
              "UUID":     res.uuid,
              "$name":    res.name,
              "$created": res.created_at,
              "$email":   res.email
            });
          }


          Session.create(res.authentication_token, res.uuid, res.role);
          // BranchIo.userCompletedAction("fb_login", null);
          // persistant storage of uuid and auth_token:
          window.localStorage.setItem("uuid", res.uuid);
          window.localStorage.setItem("token",res.authentication_token);
          window.localStorage.setItem("role",res.role);
          saveUserType(res);
          return res;
        });
    };

    authService.getUser = function (authentication_token, uuid, role) {
      Session.create(authentication_token, uuid, role);
      return User.get()
        .$promise.then(
          function (res) {
            saveUserType(res);
            return res;
          },
          function (error) {
            console.log("error login", error); // error message sent by user.js
            return $q.reject(error);
          });
    };

    authService.isAuthenticated = function () {
      return !!Session.uuid;
    };

    authService.logout = function () {
      var q = $q.defer();
      // Tell backend which user's device tokens to delete before we destroy the server on the device.
      if(window.localStorage.getItem("apns_device_token")) {
        User.removeDevice({user: {apns_device_token: window.localStorage.getItem("apns_device_token")}})
          .$promise
          .finally(function(){
            Session.destroy();
            return q.resolve();
          });
      } else if(window.localStorage.getItem("gcm_device_token")) {
        console.log("gcm remove device");
        User.removeDevice({user: {gcm_device_token: window.localStorage.getItem("gcm_device_token")}})
          .$promise
          .finally(function(){
            console.log("session destroy");
            Session.destroy();
            return q.resolve();
          });
      } else {
        Session.destroy();
        $timeout(function() { q.resolve(); }, 1);
      }
      return q.promise;
    };

    authService.isAuthorized = function (authorizedRoles) {
      if (!angular.isArray(authorizedRoles)) {
        authorizedRoles = [authorizedRoles];
      }
      return (authService.isAuthenticated() &&
        authorizedRoles.indexOf(Session.userRole) !== -1);
    };

    return authService;
  }
])

.factory('AuthInterceptor', [
  '$rootScope',
  '$q',
  'AUTH_EVENTS',
  function ($rootScope, $q, AUTH_EVENTS) {
    return {
      responseError: function (response) {
        $rootScope.$broadcast(AUTH_EVENTS.notAuthenticated);
        $rootScope.$broadcast({
          401: AUTH_EVENTS.notAuthenticated,
          403: AUTH_EVENTS.notAuthorized,
          419: AUTH_EVENTS.sessionTimeout,
          440: AUTH_EVENTS.sessionTimeout
        }[response.status], response);
        return $q.reject(response);
      }
    };
  }
]);
