angular.module('jobmapp.generics.services')

.service('GeoLocation', [
  '$q',
  'User',
  '$rootScope',
  'AUTH_EVENTS',
  '$timeout',
  function ($q, User, $rootScope, AUTH_EVENTS, $timeout) {

    var self = this;
    var updateInProgress = false;
    this.currentLocation = [];
    this.currentLocationDetails = {timestamp: 0};

    this.updateCurrentLocation = function ($scope, minSecondsSinceUpdate) {
      // set minSecondsSinceUpdate to 30 sec if not defined:
      minSecondsSinceUpdate = typeof minSecondsSinceUpdate !== 'undefined' ? minSecondsSinceUpdate : 180;
      var deferred = $q.defer();
      // seconds since last update of currentLocation:
      var timedifference = (Date.now() - this.currentLocationDetails.timestamp) / 1000;

      if (updateInProgress) { // current location update already in progress
        var destroy = $rootScope.$on('current_location_changed', function(event, data) {
          // current location event fired and cought in resolve geolocation
          destroy();
          // event destroyed
          deferred.resolve(self.currentLocation);
        })
      } else if (timedifference < minSecondsSinceUpdate) {  // Location data still up to date
        deferred.resolve(self.currentLocation);
      } else {  // Update Current Location
        updateInProgress = true;
        var options = { enableHighAccuracy: true };
        navigator.geolocation.getCurrentPosition(
          function(position) {
            oldLocation = self.currentLocation;
            self.currentLocation = [position.coords.latitude ,position.coords.longitude];
            self.currentLocationDetails = position;
            if (oldLocation != self.currentLocation) {
              $rootScope.$broadcast('current_location_changed', self.currentLocation);  //broadcast location change
            }
            if ($rootScope.currentUser) {
              var user = {position: self.currentLocation};
              User.update(user, function(returndata) {
              });
            } else {
              $rootScope.$on(AUTH_EVENTS.loginSuccess, function () {
                var user = {position: self.currentLocation};
                $timeout(function () {
                  User.update(user, function(returndata) {
                  });
                }, 500);
              });
            }
            updateInProgress = false;
            deferred.resolve(self.currentLocation);
          },
          function() {
            deferred.reject("Current Location not Found");
          },
          options
        );
      }
      return deferred.promise;
    };

    this.distanceBetween = function( latLng1, latLng2) {
      // Calculate distance in meters from latLng1 to latLng2 (straight line)
      p1 = new google.maps.LatLng(latLng1[0], latLng1[1]);
      p2 = new google.maps.LatLng(latLng2[0], latLng2[1]);
      distanceMeters = Math.round(google.maps.geometry.spherical.computeDistanceBetween(p1, p2));
      return distanceMeters;
    };

    this.distanceToCurrent = function( latLng) {
      // Calculate distance in meters from latLng to current location (straight line)
      p1 = new google.maps.LatLng(this.currentLocation[0], this.currentLocation[1]);
      p2 = new google.maps.LatLng(latLng[0], latLng[1]);
      distanceMeters = Math.round(google.maps.geometry.spherical.computeDistanceBetween(p1, p2));
      return distanceMeters;
    };

    this.distanceFromCurrentNow = function( latLng) {

      // Calculate distance in meters from latLng to current position (straight line) - return promise

      var distanceMeters = null;
      var deferred = $q.defer();

      timestamp = new Date().valueOf();
      secondsSinceLastLocation = (timestamp - currentLocationDetails.timestamp) / 1000;

      if (currentLocationDetails.timestamp && secondsSinceLastLocation < 120) {
        // use old location
        p1 = new google.maps.LatLng(currentLocation[0], currentLocation[1]);
        p2 = new google.maps.LatLng(latLng[0], latLng[1]);
        distanceMeters = Math.round(google.maps.geometry.spherical.computeDistanceBetween(p1, p2));
        deferred.resolve({distanceMeters: distanceMeters});
      } else {
        // get new location
        this.getCurrentLocation().then(function() {
          p1 = new google.maps.LatLng(currentLocation[0], currentLocation[1]);
          p2 = new google.maps.LatLng(latLng[0], latLng[1]);
          distanceMeters = Math.round(google.maps.geometry.spherical.computeDistanceBetween(p1, p2));
          deferred.resolve({distanceMeters: distanceMeters});
        });
      }
      return deferred.promise;
    };

    return this;
  }
]);
