angular.module('jobmapp.generics.services')
.factory('httpRequestInterceptor', [
  'Session',
  '$rootScope',
  'DeviceType',
  function(Session, $rootScope, DeviceType) {
    DeviceType.init();
    return {
      request: function (request) {
        request.headers = {
          'Accept': 'application/x-jobmapp-api-v1+json'
        }

        if (DeviceType.isApp()){
          request.headers['X-Auth-Method'] = 'token';
          if (Session.authentication_token && Session.authentication_token.length > 0) {
            request.headers['X-User-Uuid']    =  Session.uuid;
            request.headers['X-User-Token']   =  Session.authentication_token;
            request.headers['X-App-Platform'] =  window.ionic.Platform.platform();
            request.headers['X-App-Version']  =  $rootScope.appVersion ? $rootScope.appVersion : 'unknown';
          }
        } else if (DeviceType.isWeb()) {
          request.headers['X-Auth-Method']     = 'cookie';
          request.headers['X-App-Platform']    = 'web-app';
        }

        var pattern = /\/(?:jobs|me|services)\/(?:photo-upload|header-upload)\.json$/g;
        if (pattern.test(request.url)) {
          request.headers['Content-Type'] = undefined;
        } else {
          request.headers['Content-Type'] = 'application/json';
        }

        return request;
      },
      response: function(response){
        if ($rootScope.appVersion && typeof(response.headers()['x-app-min-version']) != 'undefined'){
          if (response.headers()['x-app-min-version'] > $rootScope.appVersion) {
            $rootScope.$broadcast('app-outdated');
          } else {
            $rootScope.$broadcast('app-not-outdated');
          }
        }
        
        return response;
      }
    }
  }
])
;
