angular.module('jobmapp.generics.services')
.factory('JobMap', [
  '$q',
  'GeoLocation',
  '$compile',
  '$ionicLoading',
  '$timeout',
  'gettextCatalog',
  'env',
  'partnerServices',
  function(
    $q,
    GeoLocation,
    $compile,
    $ionicLoading,
    $timeout,
    gettextCatalog,
    env,
    partnerServices) {

    return function() {
      var self = this;

      // used in legacy maps
      var markers = [];
      // Pins for job pages
      var jobberPageMarkers = [];
      // Pins for jobs
      var jobMarkers  = [];

      var infoWindow  = new google.maps.InfoWindow("");
      this.map        = {};
      this.searchBox  = {};
      this.mc         = {};

      this.initMap = function($scope, map_canvas, latLng, noHomeControl){
        var mapOptions = {
          zoom:               2,
          zoomControl:        false,
          center:             new google.maps.LatLng(latLng[0], latLng[1]),
          mapTypeId:          google.maps.MapTypeId.ROADMAP,
          panControl:         false,
          streetViewControl:  false,
          scaleControl:       false,
          overviewMapControl: false,
          mapTypeControl:     false
        }

        this.map            = new google.maps.Map(document.getElementById(map_canvas), mapOptions);
        var homeControlDiv  = document.createElement('div');
        var showInfo        = false;
        if (map_canvas == 'map_canvas' || map_canvas == 'map_jobberpage_location_canvas') {
          showInfo = true
        };
        if (noHomeControl != true) {
          var homeControl = new this.HomeControl($scope, homeControlDiv, showInfo);
        };

        homeControlDiv.index = 1;
        this.map
          .controls[google.maps.ControlPosition.BOTTOM_CENTER]
          .push(homeControlDiv);

        // add kml layer if service with kmlUrl
        if ($scope.job && $scope.job.specialSubCategory && $scope.job.service_id && partnerServices.get($scope.job.service_id).kmlUrl) {
          var ctaLayer = new google.maps.KmlLayer({
              url: partnerServices.get($scope.job.service_id).kmlUrl,
              map: this.map,
              clickable: false
          });
        }

        // If in "create map" add click listener (not in "search map"):
        if (map_canvas == 'map_canvas' || map_canvas == 'map_jobberpage_location_canvas') {
          google.maps.event.addListener(this.map, 'click', function(event) {
            var searchBox = document.getElementById('map_searchbox');
            if (searchBox) {
              searchBox.blur();
            }
            //cordova.plugins.Keyboard.close();
            $scope.latLng = [event.latLng.lat(), event.latLng.lng()];
            self.clearMarkers();
            self.getAddress($scope.latLng).then(function(res) {
              $scope.address          = res.formatted_address;
              $scope.address_details  = res;
              var contentString       = "<div location-info></div>";
              var compiled            = $compile(contentString)($scope);

              self.addMarkersToMap($scope.latLng, compiled, null, true);
            });
          });
        };
      };

      this.AddCurrentLocationPoint = function($scope, showText, forceUpdate, doNotCenter, noSpinner) {

        // no not show and hide spinner if noSpinner is true
        // otherwise the spinner doesnt work in the barrio map
        if (!noSpinner) {
          $ionicLoading.show({
            template:           gettextCatalog.getString("Buscando ubicación actual (GPS)"),
            hideOnStateChange:  true,
            duration:           5000
          });
        }

        if (forceUpdate) {
          var minSecondsSinceUpdate = 0;
        }
        GeoLocation.updateCurrentLocation($scope, minSecondsSinceUpdate)
          .then(function (currentLocation) {
            var currentLocationIcon =  {
              'url':        'img/my-location.png',
              'size':       new google.maps.Size(40, 40),
              'scaledSize': new google.maps.Size(20, 20),
              'origin':     new google.maps.Point(0, 0),
              'anchor':     new google.maps.Point(10, 10),
              flat: true
            };
            if (showText) {
              self.getAddress(currentLocation).then(function (res) {
                $scope.address          = res.formatted_address;
                $scope.address_details  = res;
                $scope.latLng           = currentLocation;
                var compiled            = $compile("<div location-info></div>")($scope);

                self.addMarkersToMap(
                  currentLocation,
                  compiled,
                  currentLocationIcon,
                  true );

                self.CenterCurrentLocation();
              });
            } else {
              var compiled = "";
              self.addMarkersToMap(
                currentLocation,
                compiled,
                currentLocationIcon );

              if (!doNotCenter) {
                self.CenterCurrentLocation();
              }
            }
          })
          .finally(function () {
            if (!noSpinner) {
              $ionicLoading.hide();
            }
          })
      };

      this.CenterCurrentLocation = function() {
        var latLng = new google.maps.LatLng(GeoLocation.currentLocation[0],GeoLocation.currentLocation[1]);
        this.map.setCenter(latLng);
        this.map.setZoom(14);
        var searchBounds = this.map.getBounds();
        if (this.searchBox.setBounds) {
          this.searchBox.setBounds(searchBounds);
        }
      };

      this.SetCenter = function(latLng) {
        var latLng = new google.maps.LatLng(latLng[0],latLng[1]);
        this.map.setCenter(latLng);
        this.map.setZoom(14);
      };

      this.clearMarkers = function() {
        angular.forEach([jobberPageMarkers, markers, jobMarkers], function(markerList, index){
          if (markerList) {
            angular.forEach (markerList, function(item) {
              item.setMap(null);
            });
          }
        });
        markers           = [];
        jobberPageMarkers = [];
        jobMarkers        = [];
      };

      this.addMarkersToMap = function(latLng, contentString, markerIcon, open){

        var mapBounds = new google.maps.LatLngBounds();
        var latitudeAndLongitudeOne = new google.maps.LatLng(latLng[0],latLng[1]);
        //Marker + infowindow + angularjs compiled ng-click
        if (typeof(markerIcon) == 'string') {
          markerIcon = new google.maps.MarkerImage(
            markerIcon,
            null,
            null,
            null,
            new google.maps.Size(26,40) );
        }
        var marker = new google.maps.Marker({
          position: latitudeAndLongitudeOne,
          icon:     markerIcon,
          flat:     true,
          map:      this.map,
          title:    gettextCatalog.getString('Home')
        });
        marker.infoWindowContent = contentString[0];

        if (open == true) {
          $timeout(function() {
            infoWindow.setContent(marker.infoWindowContent);
            infoWindow.open(self.map, marker);
            self.fixInfoWindow();
          }, 500);
        };

        google.maps.event.addListener(marker, 'click', function() {
          infoWindow.setContent(marker.infoWindowContent);
          infoWindow.open(this.map, marker);
          self.fixInfoWindow();
        });

        // write marker in markers array to be able to delete all markers later:
        markers.push(marker);

      };

    // ==============================================

      this.pushMarkerArray = function() {
         this.mc.addMarkers(jobberPageMarkers);
      };

      this.addJobberPageMarker = function(jobberPage, markerPathImage, callback){

        var latitudeAndLongitudeOne = new google.maps.LatLng(jobberPage.fixed_position[0], jobberPage.fixed_position[1]);
        var marker = new google.maps.Marker({
          position: latitudeAndLongitudeOne,
          icon: {
              'url': markerPathImage,
              'size': new google.maps.Size(80, 80),
              'scaledSize': new google.maps.Size(33, 40),
              'origin': new google.maps.Point(0, 0),
              'anchor': new google.maps.Point(19, 40),
              flat: true
          },
          flat: true,
          animation: google.maps.Animation.DROP,
          //map: this.map,
        });
        if (callback != null) {
          google.maps.event.addListener(marker, 'click', callback);
          google.maps.event.addListener(marker, 'click', function() {
            var latLng = new google.maps.LatLng(marker.position.lat(),marker.position.lng());
            this.map.setCenter(latLng);
            this.map.panTo(marker.position);
            this.map.setZoom(this.map.getZoom());
            marker.setAnimation(google.maps.Animation.BOUNCE);
            setTimeout(function(){ marker.setAnimation(null); }, 700);
          });
        }
        // write marker in markers array to be able to delete all markers later:
        jobberPageMarkers.push(marker);
        //this.mc.addMarker(marker);
        return marker;
      };

      this.addJobMarker = function(job, markerPathImage, callback){
        var latitudeAndLongitudeOne = new google.maps.LatLng(job.position[0],job.position[1]);
        var marker = new google.maps.Marker({
          position: latitudeAndLongitudeOne,
          icon: {
            'url': markerPathImage,
            'size': new google.maps.Size(26, 40),
            'scaledSize': new google.maps.Size(26, 40),
            'origin': new google.maps.Point(0, 0),
            'anchor': new google.maps.Point(13, 40),
            flat: true
          },
          flat:       true,
          animation:  google.maps.Animation.DROP,
          map:        this.map,
        });
        if (callback!=null) {
          google.maps.event.addListener(marker, 'click', callback);
          google.maps.event.addListener(marker, 'click', function(){
            var latLng = new google.maps.LatLng(marker.position.lat(),marker.position.lng());
            this.map.setCenter(marker.position);
            this.map.panTo(marker.position);
            this.map.setZoom(this.map.getZoom());
            marker.setAnimation(google.maps.Animation.BOUNCE);
            setTimeout(function(){ marker.setAnimation(null); }, 700);
          });
        }
        // write marker in markers array to be able to delete all markers later:
        jobMarkers.push(marker);
        return marker;
      };

      this.initMapJobberPages = function($scope, element, latLng, callback){
        var mapOptions = {
          zoom:               2,
          zoomControl:        false,
          center:             new google.maps.LatLng(latLng[0], latLng[1]),
          mapTypeId:          google.maps.MapTypeId.ROADMAP,
          panControl:         false,
          streetViewControl:  false,
          scaleControl:       false,
          overviewMapControl: false,
          mapTypeControl:     false
        }

        this.map      = new google.maps.Map(element, mapOptions);
        var showInfo  = false;
        google.maps.event.addListener(this.map, 'click', callback);

        var mcOptions = {
          gridSize: 100,
          maxZoom: 12,
          styles: [{
            height: 53,
            url: "/lib/markerclustererplus/images/m1.png",
            width: 53
          }],
        };
        this.mc = new MarkerClusterer(this.map, [], mcOptions);

        // if (noHomeControl != true) var homeControl = new this.HomeControl($scope, homeControlDiv, showInfo);
      };

      this.addMarker = function(location, kind){
        var latitudeAndLongitudeOne = new google.maps.LatLng(location[0], location[1]);
        var markerPathImage = '';
        switch ( kind ) {
          case 'job':
            var marker = new google.maps.Marker({
              position: latitudeAndLongitudeOne,
              icon: {
                'url':        "img/map_pin.png",
                'size':       new google.maps.Size(26, 40),
                'scaledSize': new google.maps.Size(26, 40),
                'origin':     new google.maps.Point(0, 0),
                'anchor':     new google.maps.Point(13, 40)
              },
              flat:       true,
              animation:  google.maps.Animation.DROP,
              map:        this.map
            });
            break;

          case 'jobber':
            var marker = new google.maps.Marker({
              position: latitudeAndLongitudeOne,
              icon: {
                'url': "img/categories/pins/defaultPin.png",
                'size': new google.maps.Size(80, 80),
                'scaledSize': new google.maps.Size(33, 40),
                'origin': new google.maps.Point(0, 0),
                'anchor': new google.maps.Point(19, 40)
              },
              flat: true,
              animation: google.maps.Animation.DROP,
              map: this.map
            });
            break;

          default:
            var marker = new google.maps.Marker({
              position: latitudeAndLongitudeOne,
              icon: {
               'url':         "img/my-location.png",
               'size':        new google.maps.Size(40, 40),
               'scaledSize':  new google.maps.Size(20, 20),
               'origin':      new google.maps.Point(0, 0),
               'anchor':      new google.maps.Point(10, 10)
              },
              flat: true,
              animation: google.maps.Animation.DROP,
              map: this.map,
            });
            break;
        }

        // write marker in markers array to be able to delete all markers later:
        markers.push(marker);
        return marker;
      };

      // =================================================================

      this.fixInfoWindow = function(){
        // This is a temporary fix of a Google Maps bug.
        // When in mobile Chrome browser mode, the close icons of infoWindows
        // are blurry. This is a fix.
        $timeout(function(){
          // We can not catch the close icon directly so we take it sibling
          // and move from that down on.
          gmStyleIws = document.getElementsByClassName("gm-style-iw")
          if (gmStyleIws.length > 0){
            window.el = gmStyleIws[0];
            _(gmStyleIws).forEach(function(el){
              if (el.parentNode.style.visibility != "hidden" && el.nextSibling != null){
                // resize icon container.
                el.nextSibling.style.width  = "13px";
                el.nextSibling.style.height = "13px";

                // Resisize and pisition sprouts.
                img               = el.nextSibling.firstChild;
                img.style.left    = "-2px";
                img.style.top     = "-336px";
                img.style.width   = "59px";
                img.style.height  = "492px";
              }
            }).value();
          };
        });
      }

      this.linkSearchBox = function($scope, inputField){

        //map.controls[google.maps.ControlPosition.BOTTOM_RIGHT].push(document.getElementById(inputField));

        this.searchBox = new google.maps.places.SearchBox(document.getElementById(inputField));

        google.maps.event.addListener(this.searchBox, 'places_changed', function() {

            //cordova.plugins.Keyboard.close();
            var places = self.searchBox.getPlaces();

            if (places.length == 0) {
              alert("no places");
              return;
            }

         // For each place, get the icon, place name, and location.
            self.clearMarkers();

            var bounds = new google.maps.LatLngBounds();
            for (var i = 0, place; place = places[i]; i++) {

              // Create a marker for each place.

              latLng                  = [place.geometry.location.lat(), place.geometry.location.lng()];
              $scope.address          = place.formatted_address;
              $scope.address_details  = place;
              $scope.latLng           = latLng;

              var compiled    = $compile("<div location-info></div>")($scope);

              self.addMarkersToMap(latLng, compiled, null, true);
              self.SetCenter(latLng);

            }

          //          map.fitBounds(bounds);
        });

        return this.searchBox;

      };

      this.getAddress = function(latLng) {
        var googleLatLng  = new google.maps.LatLng(latLng[0],latLng[1]);
        var geocoder      = new google.maps.Geocoder();
        var q             = $q.defer();
        //var searchbox = document.getElementById(inputField);
        geocoder.geocode({'latLng': googleLatLng}, function(results, status) {
          if (status == google.maps.GeocoderStatus.OK) {
            if (results[0]) {
              q.resolve(results[0]);
            } else {
              alert('No results found');
            }
          } else {
            alert('Geocoder failed due to: ' + status);
          }
        });
        return q.promise;
      };

      this.HomeControl = function ($scope, controlDiv, showInfo) {

        // Set CSS styles for the DIV containing the control
        // Setting padding to 5 px will offset the control
        // from the edge of the map

        controlDiv.style.padding = '5px';

        // Set CSS for the control border
        var controlUI = document.createElement('div');
        controlUI.style.borderWidth = '2px';
        controlUI.style.cursor      = 'pointer';
        controlUI.style.textAlign   = 'center';
        controlUI.title             = 'Click to set the map to Home';
        controlDiv.appendChild(controlUI);

        // Set CSS for the control interior
        var controlText = document.createElement('div');
        controlText.style.marginBottom  = '20px';
        controlText.innerHTML           = '<button class="button button-stable button-small icon ion-android-locate" style="border-radius: 50%; width: 50px; height: 36px; width: 36px; padding-top: 3px; border: none;"></button>';
        controlUI.appendChild(controlText);

        // Setup the click event listeners: go to current location

        google.maps.event.addDomListener(controlUI, 'click', function() {
          var searchBox = document.getElementById('map_searchbox');
          if (searchBox) {
            searchBox.blur();
            self.clearMarkers();
          }
          // add courrent location marker with forced update of current location
          self.AddCurrentLocationPoint($scope, showInfo, true);
        });

      };

      return this;
    }
    //- See more at: http://www.devx.com/wireless/implement-google-maps-api-on-phonegap-using-the-device-api.html#sthash.7MMzYFh9.dpuf

  }
])
;
