angular.module('jobmapp.generics.services')
.factory('UserExtentions', [
  'env',
  'ServiceExtentions',
  function(env, ServiceExtentions) {
    return {
      transformResponse: function(user) {
        user.jobberSince = moment(user.created_at).format('DD.MM.YYYY');
        user.photoPaths = [];
        if (user.photo_file_names) {
          if (user.photo_file_names.length > 0) {
            user.photoPaths = _.map(user.photo_file_names, function(fileName){
              return env.apiEndPoint + "/uploads/users/" + fileName;
            })
          }
        }
        user.headerPhotoPaths = [];
        if (user.header_file_names) {
          if (user.header_file_names.length > 0) {
            user.headerPhotoPaths = _.map(user.header_file_names, function(fileName){
              return env.apiEndPoint + "/uploads/user-headers/" + fileName;
            })
          }
        }

        if (user.ratings) {
          if (user.ratings.length > 0) {
            _(user.ratings).forEach(function(rating){
              if (rating.creator.image_path && rating.creator.image_path.length > 0) {
                rating.creator.thumbPath = env.apiEndPoint + "/uploads/users/thumb_" + rating.creator.image_path
              } else {
                rating.creator.thumbPath = "img/user_dark.png";
              }
            }).value();
          }
        }

        if (user.email_is_temporary) user.email = "";

        user.thumbPath = function () {
          if (user.photo_file_names.length > 0) {
            return env.apiEndPoint + "/uploads/users/thumb_" + _.last(user.photo_file_names)
          } else {
            return "img/user_dark.png";
          }
        }
        user.bigImagePath = function () {
          if (user.photo_file_names.length > 0) {
            return _.last(user.photoPaths);
          } else {
            return "img/user_dark.png";
          }
        }

        user.headerImagePath = function () {
          if (user.header_file_names.length > 0) {
            return _.last(user.headerPhotoPaths);
          } else {
            return "img/backgroundProfileBlack.png";
          }
        }

        user.initials = function () {
          var initials = '';
          if (user.name && typeof user.name == 'string' && user.name.length > 0 ) {
            var nameParts =  user.name.split(' ');
            if (nameParts.length > 0) {
              initials = initials + nameParts[0][0].toUpperCase();
              if (nameParts.length > 1) {
                initials = initials + nameParts[1][0].toUpperCase();
              }
            }
          }
          return initials;
        }

        // Create empty jobber_page unless it already exists.
        user.jobber_page = user.jobber_page || {
          fixed_position: null,
          address: null,
          business_name: null,
          username: null
        };

        // Create empty jobber_page unless it already exists.
        user.internal_review = user.internal_review || {
          id: null,
          stars: null,
          comment: null,
          created_at: null,
          updated_at: null
        };
        // user.internal_review.stars = 3;

        user.getFullUrl = function(){
          if(this.jobber_page && this.jobber_page.username) {
            return env.webRoot + "/"          + this.jobber_page.username
          } else {
            return env.webRoot + "/jobbers/"  + this.uuid
          }
        }
        user.getPrettyUrl = function(){
          return this.getFullUrl().replace(/http(s){0,1}:\/\//, "");
        }

        if (user.jobber_page && user.jobber_page.services && user.jobber_page.services.length > 0) {
          _(user.jobber_page.services).forEach(function(service){
            // Input type="number" fields need to be populated with floats. So make sure, that the prices are floats.
            service = ServiceExtentions.transformResponse(service);
          }).value();
        };

        return user;
      }
    }
  }
]);
