// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.services' is found in services.js
// 'starter.controllers' is found in controllers.js
angular.module('starter', [
  'ionic',
  'ngIOS9UIWebViewPatch',
  'starter.controllers',
  'starter.services',

  'jobmapp.generics.directives',
  'jobmapp.generics.filters',
  'jobmapp.generics.services',
  'jobmapp.generics.resources',
  'jobmapp.generics.constants',

  'ngCordova',
  'ngResource',
  'ionic.rating',
  'mlz.backbutton',
  'growingInput',
  'mlz.openurl',
  'gettext',
  'angular.filter',
  'ngMessages' ]
).run(function($ionicPlatform, $timeout, PushNotify, MessageRouter, $rootScope, $cordovaAppVersion, gettextCatalog, env, DeviceType, Session) {
  console.log("Check 1");  
  DeviceType.init();
  Session.init({
    afterCreateHook: function(){
      $ionicPlatform.ready(function() {
        // This timeout fixes a weird bug in iOS.
        $timeout(function() {
          console.log("call push init");
          PushNotify.init();
        }, 1);
        // BranchIo.setIdentity({uuid: uuid});
        $timeout(function() {
          // "this" will reference Session:
          // BranchIo.setIdentity(this.uuid, null);
        }, 1);
      });
    },
    afterDestroyHook: function(){
      // BranchIo.logout();
    }
  });
  $ionicPlatform.ready(function() {
    console.log("Check 2")
    // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
    // for form inputs)
    if(window.cordova && window.cordova.plugins.Keyboard) {
      cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
    }
    if(window.StatusBar) {
      // org.apache.cordova.statusbar required
      StatusBar.styleDefault();
    }

    if (ionic.Platform.isAndroid() || ionic.Platform.isIOS()) {
      console.log("Check 3");
      cordova.getAppVersion().then(function (version) {
        $rootScope.appVersion = version;
      });
    } else {
      $rootScope.appVersion = "browser";
    }

    gettextCatalog.baseLanguage = 'es';
    $rootScope.$broadcast('init-ui-language');

    if (env.name == "production") {
      gettextCatalog.debug = false;
    } else {
      gettextCatalog.debug = true;
    }

    if (ionic.Platform.isAndroid() || ionic.Platform.isIOS()) {
      console.log("Check 4")
      setTimeout(function() {
        navigator.splashscreen.hide();
      }, 1000);
    }

  });
})

.config(function($stateProvider, $urlRouterProvider) {


  // openFB Facebook App ID (from Facebook Developer Account)
  console.log("Check 5");

  openFB.init({appId: '297088137157787', tokenStore: window.localStorage}); //, tokenStore: '$root.fbToken'}


  // Ionic uses AngularUI Router which uses the concept of states
  // Learn more here: https://github.com/angular-ui/ui-router
  // Set up the various states which the app can be in.
  // Each state's controller can be found in controllers.js
  $stateProvider

    // setup an abstract state for the tabs directive
    .state('tab', {
      url: "/tab",
      abstract: true,
      templateUrl: "templates/tabs.html"
    })

    // CREATE JOB ===================================

    .state('tab.createjob', {
      url: '/createjob',
      views: {
        'tab-createjob': {
          templateUrl: 'templates/tab-createjob-category.html',
          controller: 'CreateJobCategoryCtrl'
        }
      },
      resolve: {
        jobFormData: function(jobForm){
          return jobForm.getDraft();
        }
      }
    })

    // SEARCH JOB

    .state('tab.searchjob', {
      url: '/searchjob',
      views: {
        'tab-searchjob': {
          templateUrl: 'templates/tab-searchjob.html',
          controller: 'SearchJobCtrl'
        }
      }
    })
    .state('tab.searchjob.location', {
      url: '/location/:id',
      views: {
        'tab-searchjob@tab': {
          templateUrl: 'templates/tab-showjob-location.html',
          controller: 'ShowJobLocationCtrl'
        }
      },
      data: {
        map_canvas : 'map_showjob_canvas'
      }
    })

     // LOCAL

    .state('tab.local', {
      url: '/local',
      views: {
        'tab-local': {
          templateUrl: 'templates/tab-local.html',
          controller: 'LocalCtrl'
        }
      }
    })


     // Directory

    .state('tab.directory', {
      url: '/directory',
      views: {
        'tab-directory': {
          templateUrl: 'templates/tab-directory.html',
          controller: 'DirectoryController'
        }
      }
    })
    .state('tab.directory.search', {
      url: '/search',
      views: {
        'tab-directory@tab': {
          templateUrl: 'templates/directory/search.html',
          controller: 'DirectoryController',
          resolve: {
            records: function($rootScope, Search){
              var defaultSearch = {
                lat: 39.5735,
                lng: 2.6492,
              }
              console.log("rootscope", $rootScope);
              if ($rootScope.currentUser && $rootScope.currentUser.position) {
                defaultSearch["lat"] = $rootScope.currentUser.position[0];
                defaultSearch["lng"] = $rootScope.currentUser.position[1];
              }
              return Search.search(defaultSearch).$promise;
            }
          }
        }
      }
    })




    // MY JOBS

    .state('tab.myjobs', {
      url: '/myjobs',
      views: {
        'tab-myjobs': {
          templateUrl: 'templates/tab-myjobs.html',
          controller: 'MyJobsCtrl'
        }
      }
    })

    .state('tab.myjobs.selectjobber', {
      url: '/selectjobber/:id',
      params: {selectJobberId: null},
      parent: 'tab.myjobs',
      views: {
        'tab-myjobs@tab': {
          templateUrl: 'templates/tab-myjobs-select-jobber.html',
          controller: 'SelectJobberCtrl',
          resolve: {
            job: function($stateParams, Job){
              return Job.get({id: $stateParams.id}).$promise;
            }
          }
        }
      }
    })

    .state('tab.myjobs.selectjobber.edit', {
      url: '/edit',
      parent: 'tab.myjobs.selectjobber',
      views: {
        'tab-myjobs@tab': {
          templateUrl: 'templates/tab-createjob-details.html',
          controller: 'CreateJobCtrl',
          resolve: {
            jobFormData: function(jobForm){
              return jobForm.getToUpdate();
            }
          }
        }
      }
    })

    .state('tab.myjobs.selectjobber.edit.photos', {
      url: '/photos',
      views: {
        'tab-myjobs@tab': {
          templateUrl: 'templates/tab-createjob-photos.html',
          controller: 'CreateJobPhotoCtrl',
          resolve: {
            jobFormData: function(jobForm){
              return jobForm.getToUpdate();
            }
          }
        }
      }
    })

    .state('tab.myjobs.selectjobber.edit.description', {
      url: '/description',
      views: {
        'tab-myjobs@tab': {
          templateUrl: 'templates/tab-createjob-description.html',
          controller: 'CreateJobDescriptionCtrl',
          resolve: {
            jobFormData: function(jobForm){
              return jobForm.getToUpdate();
            }
          }
        }
      }
    })

    .state('tab.myjobs.selectjobber.edit.price', {
      url: '/price',
      views: {
        'tab-myjobs@tab': {
          templateUrl: 'templates/tab-createjob-price.html',
          controller: 'CreateJobPriceCtrl',
          resolve: {
            jobFormData: function(jobForm){
              return jobForm.getToUpdate();
            }
          }
        }
      }
    })

    .state('tab.myjobs.selectjobber.edit.date', {
      url: '/date',
      views: {
        'tab-myjobs@tab': {
          templateUrl: 'templates/tab-createjob-date.html',
          controller: 'CreateJobCtrl',
          resolve: {
            jobFormData: function(jobForm){
              return jobForm.getToUpdate();
            }
          }
        }
      }
    })

    .state('tab.myjobs.selectjobber.edit.location', {
      url: '/location',
      views: {
        'tab-myjobs@tab': {
          templateUrl: 'templates/tab-createjob-location.html',
          controller: 'CreatejobLocationCtrl',
          resolve: {
            jobFormData: function(jobForm){
              return jobForm.getToUpdate();
            }
          }
        }
      }
    })

    .state('tab.myjobs.confirmedjob', {
      url: '/confirmedjob/:id',
      views: {
        'tab-myjobs@tab': {
          templateUrl: 'templates/tab-myjobs-confirmed-job.html',
          controller: 'SelectJobberCtrl',
          resolve: {
            job: function($stateParams, Job){
              return Job.get({id: $stateParams.id}).$promise;
            }
          }
        }
      }
    })

    .state('tab.myjobs.applieddetails', {
      url: '/applieddetails/:id',
      views: {
        'tab-myjobs@tab': {
          templateUrl: 'templates/tab-myjobs-applied-details.html',
          controller: 'MyJobsAppliedDetailsCtrl',
          resolve: {
            job: function($stateParams, Job){
              return Job.get({id: $stateParams.id}).$promise;
            }
          }
        }
      }
    })

    .state('tab.myjobs.location', {
      url: '/location/:id',
      views: {
        'tab-myjobs@tab': {
          templateUrl: 'templates/tab-myjobs-location.html',
          controller: 'ShowJobLocationCtrl'
        }
      },
      data: {
        map_canvas : 'map_myjob_canvas'
      }
    })


    // EDIT PROFILE

    .state('tab.account', {
      url: '/account',
      params: {referringUser: null},
      views: {
        'tab-account': {
          templateUrl: 'templates/tab-account.html',
          controller: 'AccountCtrl'
        }
      },
      data: {
        map_canvas : 'map_profile_canvas'
      }
    })

    .state('tab.account.settings', {
      url: '/settings',
      params: {call: null},
      views: {
        'tab-account@tab': {
          templateUrl: 'templates/tab-account-settings.html',
          controller: 'SettingsCtrl'
        }
      }
    })


    .state('tab.account.jobberpage', {
      url: '/jobberpage',
      views: {
        'tab-account@tab': {
          templateUrl: 'templates/tab-account-jobberpage.html',
          controller: 'JobberpageCtrl'
        }
      },
      data: {
        map_canvas : 'map_jobberpage_canvas'
      },
      resolve: {
        me: function(User) {
          return User.get().$promise;
        }
      }
    })

    .state('tab.account.jobberpage.basic', {
      url: '/basic',
      views: {
        'tab-account@tab': {
          templateUrl: 'templates/tab-account-jobberpage-basic.html',
          controller: 'JobberpageCtrl'
        }
      }
    })

    // not done yet: ==============
    // .state('tab.chat', {
    //   url: '/chat',
    //   params: {referringUser: null},
    //   views: {
    //     'tab-account': {
    //       templateUrl: 'templates/tab-chat.html',
    //       //controller: 'AccountCtrl'

    .state('tab.account.jobberpage.contact', {
      url: '/contact',
      views: {
        'tab-account@tab': {
          templateUrl: 'templates/tab-account-jobberpage-contact.html',
          controller: 'JobberpageCtrl'
        }
      }
    })

    .state('tab.account.jobberpage.insurance', {
      url: '/insurance',
      params: {call: null},
      views: {
        'tab-account@tab': {
          templateUrl: 'templates/tab-account-jobberpage-insurance.html',
          controller: 'JobberpageCtrl'
        }
      }
    })

    .state('tab.account.jobberpage.business', {
      url: '/business',
      params: {call: null},
      views: {
        'tab-account@tab': {
          templateUrl: 'templates/tab-account-jobberpage-business.html',
          controller: 'JobberpageCtrl'
        }
      }
    })

    .state('tab.account.jobberpage.location', {
      url: '/location',
      params: {call: null},
      views: {
        'tab-account@tab': {
          templateUrl: 'templates/tab-account-jobberpage-location.html',
          controller: 'JobberpageLocationCtrl'
        }
      },
      data: {
        map_canvas : 'map_jobberpage_location_canvas'
      }

    })

    .state('tab.account.jobberpage.service', {
      url: '/services/:id',
      views: {
        'tab-account@tab': {
          templateUrl: 'templates/tab-account-service.html',
          controller: 'ServiceCtrl'
        }
      },
      resolve: {
        service: function($stateParams, Service) {
          if ($stateParams.id == 'new') {
            return {
              description: {},
              photoServerPathByName: {},
              photoThumbServerPathByName: {},
              photo_file_names: []
            };
          } else {
            return Service.get({id: $stateParams.id}).$promise;
          }
        }
      }
    })

    .state('tab.account.jobberpage.service.category', {
      url: '/category',
      views: {
        'tab-account@tab': {
          templateUrl: 'templates/tab-account-service-category.html',
          controller: 'ServiceCtrl'
        }
      }
    })

    .state('tab.account.jobberpage.service.title', {
      url: '/title',
      views: {
        'tab-account@tab': {
          templateUrl: 'templates/tab-account-service-title.html',
          controller: 'ServiceCtrl'
        }
      }
    })

    .state('tab.account.jobberpage.service.description', {
      url: '/description',
      views: {
        'tab-account@tab': {
          templateUrl: 'templates/tab-account-service-description.html',
          controller: 'ServiceCtrl'
        }
      }
    })

    .state('tab.account.jobberpage.service.photos', {
      url: '/photos',
      views: {
        'tab-account@tab': {
          templateUrl: 'templates/tab-account-service-photos.html',
          controller: 'ServiceCtrl'
        }
      }
    })

    .state('tab.account.jobberpage.service.price', {
      url: '/price',
      views: {
        'tab-account@tab': {
          templateUrl: 'templates/tab-account-service-price.html',
          controller: 'ServiceCtrl'
        }
      }
    })


    .state('tab.account.skills', {
      url: '/skills',
      views: {
        'tab-account@tab': {
          templateUrl: 'templates/tab-account-skills.html',
          controller: 'SkillsCtrl'
        }
      }
    })
    ;


    // Generate equal substate for three of the main tabs

  angular.forEach([
    'tab.searchjob',
    'tab.local',
    'tab.directory',
    'tab.myjobs'
  ], function(statePrefix){
    // ------------------------
    // Jobberdetails / Profiles
    // ------------------------
    var stateName = statePrefix + '.jobberdetails';
    var tabName = 'tab-' + statePrefix.split('.')[1] + '@tab';
    var state = {
      url: '/jobberdetails/:candidateId',
      views: {},
      data: {
        map_canvas : 'map_jobberdetails_canvas'
      }
    };
    state.views[tabName] = {
      templateUrl: 'templates/tab-local-jobber.html',
      controller: 'MyJobsJobberdetailsCtrl',
      resolve: {
        user: function($stateParams,$ionicLoading, User) {
          $ionicLoading.show({templateUrl : 'templates/loading-animation.html', hideOnStateChange: true, duration: 10000});
          return User.getOther({id: $stateParams.candidateId}).$promise.finally(
            function() {
              $ionicLoading.hide();
            }
          );
        }
      }
    }
    // Finally add the generated state to $stateProvider.
    $stateProvider.state(stateName, state);

    // ----------------
    // Displaying a job
    // ----------------
    var stateName = statePrefix + '.showjob';
    var tabName = 'tab-' + statePrefix.split('.')[1] + '@tab';
    var state = {
      //parent: 'tab.searchjob',
      url: '/showjob/:id',
      params: {'call': null},
      views: {}
    };
    state.views[tabName] = {
      templateUrl: 'templates/tab-showjob.html',
      controller: 'ShowJobCtrl',
      resolve: {
        job: function($stateParams, Job, $ionicLoading){
          $ionicLoading.show({templateUrl : 'templates/loading-animation.html', hideOnStateChange: true, duration: 10000});
          return Job.get({id: $stateParams.id}).$promise.finally(
            function() {
              $ionicLoading.hide();
            }
          );
        }
      }
    };

    // Finally add the generated state to $stateProvider.
    $stateProvider.state(stateName, state);

    // --------------------
    // Displaying a service
    // --------------------
    var stateName = statePrefix + '.service';
    var tabName = 'tab-' + statePrefix.split('.')[1] + '@tab';
    var state = {
    //parent: 'tab.searchjob',
      url: '/services/:serviceId',
      params: {'call': null},
      views: {}
    };
    state.views[tabName] = {
      templateUrl: 'templates/services/show.html',
      controller: 'ServiceViewCtrl',
      resolve: {
        service: function($stateParams, $ionicLoading, Service){
          $ionicLoading.show({templateUrl : 'templates/loading-animation.html', hideOnStateChange: true, duration: 10000});
          return Service.get({id: $stateParams.serviceId}).$promise.then(
            function(service){ return service },
            function(){ return null }
          ).finally(
            function() {
              $ionicLoading.hide();
            }
          )
        }
      }
    };

    // Finally add the generated state to $stateProvider.
    $stateProvider.state(stateName, state);
  });

  angular.forEach([
    'tab.searchjob.showjob',
    'tab.searchjob.jobberdetails',
    'tab.local.showjob',
    'tab.local.jobberdetails',
    'tab.myjobs.showjob',
    'tab.myjobs.jobberdetails',
    'tab.myjobs.selectjobber',
    'tab.myjobs.confirmedjob',
    'tab.account'
  ], function(statePrefix){
    var stateName = statePrefix + '.showlocation';
    var tabName = 'tab-' + statePrefix.split('.')[1] + '@tab';
    var state = {
      url: '/showlocation',
      views: {},
      params: {
        kind: null,
        objectWithPosition: null,
      },
      data: {
        map_canvas: 'map_myjob_canvas'
      }
    };
    state.views[tabName] = {
      templateUrl: 'templates/tab-myjobs-location.html',
        controller: 'ShowLocationCtrl'
    }

    // Finally add the generated state to $stateProvider.
    $stateProvider.state(stateName, state);
  })

  angular.forEach([
    'tab.createjob',
    'tab.searchjob',
    'tab.local',
    'tab.directory',
    'tab.myjobs'
  ], function(statePrefix){
    // ------------------------
    // Create job details
    // ------------------------
    var stateName = statePrefix + '.details';
    var tabName = 'tab-' + statePrefix.split('.')[1] + '@tab';
    var state = {
      url: '/details/:jobCategoryID',
      views: {},
      params: {specialSubCategory: null},
      resolve: {
        jobFormData: function(jobForm){
          return jobForm.getDraft();
        }
      }
    };
    state.views[tabName] = {
      templateUrl: 'templates/tab-createjob-details.html',
      controller: 'CreateJobCtrl'
    }

    // Finally add the generated state to $stateProvider.
    $stateProvider.state(stateName, state);


    // ------------------------
    // Create job description
    // ------------------------
    var stateName = statePrefix + '.details.description';
    var tabName = 'tab-' + statePrefix.split('.')[1] + '@tab';
    var state = {
      url: '/description',
      views: {}
    };
    state.views[tabName] = {
      templateUrl: 'templates/tab-createjob-description.html',
      controller: 'CreateJobDescriptionCtrl',
    }

    // Finally add the generated state to $stateProvider.
    $stateProvider.state(stateName, state);


    // ------------------------
    // Create job price
    // ------------------------
    var stateName = statePrefix + '.details.price';
    var tabName = 'tab-' + statePrefix.split('.')[1] + '@tab';
    var state = {
      url: '/price',
      views: {}
    };
    state.views[tabName] = {
      templateUrl: 'templates/tab-createjob-price.html',
      controller: 'CreateJobPriceCtrl'
    }

    // Finally add the generated state to $stateProvider.
    $stateProvider.state(stateName, state);


    // ------------------------
    // Create job date
    // ------------------------
    var stateName = statePrefix + '.details.date';
    var tabName = 'tab-' + statePrefix.split('.')[1] + '@tab';
    var state = {
      url: '/date',
      views: {}
    };
    state.views[tabName] = {
      templateUrl: 'templates/tab-createjob-date.html',
      controller: 'CreateJobCtrl'
    }


    // Finally add the generated state to $stateProvider.
    $stateProvider.state(stateName, state);


    // ------------------------
    // Create job location
    // ------------------------
    var stateName = statePrefix + '.details.location';
    var tabName = 'tab-' + statePrefix.split('.')[1] + '@tab';
    var state = {
      url: '/location',
      views: {},
      params: {locationNumber: 1},
    };
    state.views[tabName] = {
      templateUrl: 'templates/tab-createjob-location.html',
      controller: 'CreatejobLocationCtrl'
    }

    // Finally add the generated state to $stateProvider.
    $stateProvider.state(stateName, state);


    // ------------------------
    // Create job photos
    // ------------------------
    var stateName = statePrefix + '.details.photos';
    var tabName = 'tab-' + statePrefix.split('.')[1] + '@tab';
    var state = {
      url: '/photos',
      views: {}
    };
    state.views[tabName] = {
      templateUrl: 'templates/tab-createjob-photos.html',
      controller: 'CreateJobPhotoCtrl'
    }

    // Finally add the generated state to $stateProvider.
    $stateProvider.state(stateName, state);


    // ------------------------
    // Create job cleaning
    // ------------------------
    var stateName = statePrefix + '.details.cleaning';
    var tabName = 'tab-' + statePrefix.split('.')[1] + '@tab';
    var state = {
      url: '/cleaning',
      views: {}
    };
    state.views[tabName] = {
      templateUrl: 'templates/tab-createjob-cleaning.html',
      controller: 'CreateJobCleaningCtrl'
    }

    // Finally add the generated state to $stateProvider.
    $stateProvider.state(stateName, state);
  });



  // if none of the above states are matched, use this as the fallback
  var uuid = window.localStorage.getItem("uuid");
  if (uuid) {
    $urlRouterProvider.otherwise('/tab/directory');
  } else {
    $urlRouterProvider.otherwise('/tab/account');
  };

})

.config([
  '$httpProvider',
  function ($httpProvider) {
    $httpProvider.interceptors.push('httpRequestInterceptor');
  }
])

.config(function($ionicConfigProvider) {
  $ionicConfigProvider.backButton.text('');
  $ionicConfigProvider.backButton.icon('ion-ios-arrow-left');
  $ionicConfigProvider.backButton.previousTitleText(false);
  $ionicConfigProvider.tabs.position('bottom');
})

.config(function ($compileProvider){
  $compileProvider.aHrefSanitizationWhitelist(/^\s*(https?|ftp|mailto|file|geo|tel):/);
})
;

// LOGIN DIALOG DIRECTIVE DISABLED; WE WORK WITH A MANUAL LOGIN POPUP INESTEAD
// KEEP IT FOR THE MOMENT HERE AS THIS WOULD BE A BETTER SOLUTION
/*
.config(function ($httpProvider) {
  $httpProvider.interceptors.push([
    '$injector',
    function ($injector) {
      return $injector.get('AuthInterceptor');
    }
  ]);
})
*/


/* Deep links - Remember to add the following lines to jobMapp-Info.plist file for iOS:
  <key>CFBundleURLSchemes</key>
  <array>
    <string>fb297088137157787</string>
    <string>jobmapp</string>
  </array>
*/

if ('cordova' in window) {
  // Create a sticky event for handling the app being opened via a custom URL
  cordova.addStickyDocumentEventHandler('handleurl');
}

function handleOpenURL(url) {
  console.log("entry", url);
  setTimeout(function() {
   cordova.fireDocumentEvent('handleurl', { url: url });
  }, 0);
}


;
