angular.module('starter.controllers')

// ============================ General App Controller ============================
// Available throughout APP as it gets included in BODY Tag
// Use this for persistent variable storage between the Tabs and for persistant storage when app closes

.controller('AppCtrl', function ($scope, $rootScope, $ionicPopup, $q, $ionicLoading, $ionicModal, $cordovaNetwork,$ionicScrollDelegate, Job, AUTH_EVENTS, USER_ROLES, AuthService, GeoLocation, User, $cordovaSplashscreen, $ionicHistory, $state, $ionicPlatform, gettextCatalog, $ionicSideMenuDelegate, $timeout, mixpanelService) {

  // get persistant session variables

  // $cordovaSplashscreen.show();

  var uuid = window.localStorage.getItem("uuid");
  var token = window.localStorage.getItem("token");
  var role = window.localStorage.getItem("role");


  $rootScope.currentUser = null;
  $scope.userRoles = USER_ROLES;
  $scope.isAuthorized = AuthService.isAuthorized;
  $scope.onDevice = (ionic.Platform.isAndroid() || ionic.Platform.isIOS());
  mixpanelService.init();

  $scope.isCreator = function(){
    var answer;
    if ($rootScope.currentUser) {
      answer = $rootScope.currentUser.is_creator;
    } else {
      answer = (window.localStorage.getItem('is_creator') == "true");
    }
    return answer;
  };

  $scope.isJobber = function(){
    var answer;
    if ($rootScope.currentUser) {
      answer = $rootScope.currentUser.is_jobber;
    } else {
      answer = (window.localStorage.getItem('is_jobber') == "true");
    }
    return answer;
  };

  $scope.getLanguage = function(){
    var lang;
    if ($rootScope.currentUser) {
      lang = $rootScope.currentUser.language;
    } else {
      lang = window.localStorage.getItem('language');
    }
    if(lang) {
      return lang;
    } else {
      return 'es';
    }
  };

  $scope.setIsCreator = function(saveToServer){
    var saveIt;
    if (typeof saveToServer == "undefined"){
      saveIt = true;
    } else {
      saveIt = saveToServer;
    }
    window.localStorage.setItem("is_creator", true);
    $ionicHistory.clearCache();

    if (saveIt && $rootScope.currentUser) {
      User.update({is_creator: true}, function(returndata) {
        console.log("User was saved as job poster (creator) to server.");
      })
    }
  }

  $scope.setIsJobber = function(saveToServer){
    var saveIt;
    if (typeof saveToServer == "undefined"){
      saveIt = true;
    } else {
      saveIt = saveToServer;
    }
    window.localStorage.setItem("is_jobber", true);
    $ionicHistory.clearCache();

    if (saveIt && $rootScope.currentUser) {
      User.update({is_jobber: true}, function(returndata) {
        console.log("User was saved as jobber to server.");
      })
    }
  }

  $scope.setLanguage = function(lang, saveToServer){
    var saveIt;
    if (typeof saveToServer == "undefined"){
      saveIt = true;
    } else {
      saveIt = saveToServer;
    }
    if ($rootScope.currentUser) {
      $rootScope.currentUser.language = lang;
    }
    window.localStorage.setItem("language", lang);
    $scope.initUiLanguage();

    if (saveIt && $rootScope.currentUser) {
      User.update({language: lang}, function(returndata) {
        console.log("User's language was saved to server.");
      })
    }
  }

  $scope.initUiLanguage = function() {
    var lang = $scope.getLanguage();
    gettextCatalog.currentLanguage = lang;
    gettextCatalog.setCurrentLanguage(lang);
    moment.locale(lang);
    //Problem with clearCache in tab.createjob when select language
    if($state.current.name != "tab.createjob") {
      $ionicHistory.clearCache();
    }
  }

  $scope.openNotify = function() {
    console.log("toggle");
    $ionicSideMenuDelegate.toggleRight();
  };


  // Create session and get user automatically:

  if (uuid && token && role && uuid.length > 0 && token.length > 0) {
    $ionicLoading.show({template: "Login", hideOnStateChange: true, duration: 10000});
    AuthService.getUser(token, uuid, role).then(function (user) {
      $ionicLoading.hide();
      $rootScope.$broadcast(AUTH_EVENTS.loginSuccess);
      $scope.setCurrentUser(user);
    }, function () {
      console.log("auto-login failed");
      $ionicLoading.show({template: "Login no ha sido posible", hideOnStateChange: true, duration: 2000});
      $rootScope.$broadcast(AUTH_EVENTS.loginFailed);
    });
  }

  // initiate GeoLocation service and check on local availability:
  $ionicPlatform.ready(function() {
    GeoLocation.updateCurrentLocation($scope).then(function() {
      // if ((GeoLocation.distanceToCurrent([39.56, 2.76]) / 1000) > 100) {
      //   var alertPopup = $ionicPopup.alert({
      //     title: 'jobMapp Mallorca',
      //     template: 'Estamos comencando el servicio de jobMapp en Mallorca. La app funciona en otros lugares pero la publicidad y disponibilidad de jobbers se concentra de momento en la isla.'
      //   });
      // }
    });
  });

  $rootScope.$on('init-ui-language', function(){
    $scope.initUiLanguage();
  });

  // Listen on device token changes
  $rootScope.$on('gcm_device_token-received', function(){
    if($rootScope.currentUser){
      // console.log("user already logged in so we can simply update her gcm_device_token");
      // Tell backend the regid so that it can send notifications to this device.
      User.update({
        user:{
          gcm_device_token: window.localStorage.getItem("gcm_device_token")
        }
      });
    } else {
      // console.log("user not logged in so we have to wait until user is logged in.");
      $rootScope.$on(AUTH_EVENTS.loginSuccess, function(){
        //console.log("deferred login after gcm token received");
        User.update({
          user:{
            gcm_device_token: window.localStorage.getItem("gcm_device_token")
          }
        });
      })
    }
  });

  // Listen on device token changes
  $rootScope.$on('apns_device_token-received', function(){
    // console.log("gcm_device_token-received callback");
    if($rootScope.currentUser){
      // console.log("user already logged in so we can simply update her gcm_device_token");
      // Tell backend the regid so that it can send notifications to this device.
      User.update({
        user:{
          apns_device_token: window.localStorage.getItem("apns_device_token")
        }
      });
    } else {
      // console.log("user not logged in so we have to wait until user is logged in.");
      $rootScope.$on(AUTH_EVENTS.loginSuccess, function(){
        //console.log("deferred login after gcm token changed");
        User.update({
          user:{
            apns_device_token: window.localStorage.getItem("apns_device_token")
          }
        });
      })
    }
  });
  var openAppStore = function(){
    if (ionic.Platform.isAndroid()) {
      console.log("send to Play Store");
      cordova.plugins.market.open('com.jobmapp.cordova');
    }
    if (ionic.Platform.isIOS()) {
      console.log("send to App Store");
      cordova.plugins.market.open('jobmapp');
    }
  };

  var askUserToUpdate = function(){
    if ($rootScope.updateDialogueOpen == true) {
      return
    }
    $rootScope.updateDialogueOpen = true;

    var alertPopup = $ionicPopup.show({
      title: '¡Actualiza jobMapp!',
      template: '<p>Hay una nueva versión de jobMapp. Por favor descárgala.</p>',
      buttons: [
        { text: 'Más tarde',
          onTap: function(e) {
            return false;
          }
        },
        {
          text: '<b>Actualizar</b>',
          type: 'button-positive',
          onTap: function(e) {
            return true;
          }
        }
      ]
    });

    alertPopup.then(function(answer) {
      if (answer) {
        // User confirmed.
        openAppStore();
      } else {
        // User denied
        console.log("user denied updating now");
        window.localStorage.setItem('app_outdated_warning_date', moment(new Date()).format());
      }
      $rootScope.updateDialogueOpen = false;
    });

  };

  $rootScope.$on('app-outdated', function(){
    console.log("app outdated event handler");
    var sinceString = window.localStorage.getItem('app_outdated_since');
    console.log("sinceString", sinceString);
    if (sinceString) {
      var sinceDate = moment(sinceString);
      var nowDate = moment(new Date());
      console.log(sinceDate, nowDate);
      if (nowDate.diff(sinceDate, 'days') > 3) {
        // More than 3 days are gone since warning the user. So let's force her to update.
        console.log("User has been knowing for more than 3 days that the app is outdated.");
        openAppStore();
      } else {
        console.log("User has been knowing for less than 3 days that the app is outdated.");
        var warningString = window.localStorage.getItem('app_outdated_warning_date');
        if (!warningString || moment(new Date()).diff(moment(warningString), 'minutes') > 60) {
          askUserToUpdate();
        } else {
          console.log("it is not yet 1 hour ago since last warning.");
        }
      }
    } else {
      console.log("Set app_outdated_since for the first time");
      window.localStorage.setItem('app_outdated_since', moment(new Date()).format());
      askUserToUpdate();
    }
  });

  $rootScope.$on('app-not-outdated', function(){
    window.localStorage.removeItem('app_outdated_since');
  });

  // This is used to set $rootScope.currentUser:
  $scope.setCurrentUser = function (user) {
    $rootScope.currentUser = user;
  };

  // Keep MyJobView persistent throughout App
  var activeMyJobButton = window.localStorage.getItem("activeMyJobButton");

  if (activeMyJobButton) {
    $scope.activeMyJobButton = activeMyJobButton
  }
  else {
    $scope.activeMyJobButton = 1
  };

  $scope.setMyJobView = function(view) {
    $scope.activeMyJobButton = view;
    window.localStorage.setItem("activeMyJobButton", view);
  };



// LOGIN POPOUP CAN THIS BE MOVED TO LOGIN CTRL?

  $scope.loginPopup = {};

  $scope.showLoginPopup = function() {
    $scope.data = {};
    $scope.loginPopup = $ionicPopup.show({
     templateUrl: 'templates/_login.html',
     title: 'Login',
     subTitle: '<p>Please login or sign up to jobMapp to proceed</p>',
     scope: $scope,
     buttons: [
       { text: 'Cancelar' }
     ]
    });
  };

  // Network connection check when running on device only:

  $scope.isOffline = false;
  var deviceOffline = function() {
    $scope.isOffline = true;
    $scope.$apply();
  }
  var deviceOnline = function() {
    $scope.isOffline = false;
    $scope.$apply();
  }
  if ($scope.onDevice) {
    document.addEventListener("offline", deviceOffline, false);
    document.addEventListener("online", deviceOnline, false);
  };

  $rootScope.$on('open-private-job-chat', function(event, options) {
    $scope.chat = $scope.chat || {};
    $scope.viewScope = function(){
      return $scope;
    }


    var deferred = $q.defer();

    if (options.job) {
      // The job is already loaded.
      $scope.chat.job = options.job;
      deferred.resolve();
    } else {
      if (options.jobToasterFn && !($scope.chatModal && $scope.chatModal.jobId == options.jobId)) {
        // Just throw in a jobToaster and leave this place :-)
        options.jobToasterFn("", options.message);
        return;
      } else {
        // The job needs to be loaded.
        Job.get({id: options.jobId}).$promise.then(function(job){
          $scope.chat.job = job;
          deferred.resolve();
        })
      }
    }

    if (options.toId) $scope.chat.toId = options.toId;

    deferred.promise.then(function(){
      // In case there is already a modal open, just leave it open if it the same chat.
      // And if it is a different chat, close it!
      console.log("$scope.chatModal", $scope.chatModal);
      if ($scope.chatModal) {
        if (options.job && options.job.id == $scope.chatModal.jobId) {
          // $scope.$broadcast('job.refreshed');
          return
        } else if (options.jobId && options.jobId == $scope.chatModal.jobId) {
          $scope.$broadcast('new-chat-message');
          return;
        } else {
          if ($scope.closePrivateJobChat) {
            $scope.closePrivateJobChat();
          }
        }
      }

      $ionicModal.fromTemplateUrl('templates/_chat.html', {
        scope: $scope,
        backdropClickToClose: false,
        animation: 'slide-in-up'
      })
      .then(function (modal) {

        $scope.chatModal = modal;
        $scope.chatModal.jobId = $scope.chat.job.id;

        var removeListener = $rootScope.$on('$stateChangeStart', function(event, toState, toParams, fromState, fromParams){
          $scope.closePrivateJobChat();
        });

        $scope.closePrivateJobChat = function() {
          $scope.chatModal.hide();
          $scope.chatModal.remove();
          $scope.chatModal = null;
          removeListener();
        };

        $scope.chatModal.show().then(function(){
          $scope.$broadcast('job.refreshed');
        });
      });
    })
  })

  $scope.defaultCurrency = function(){
    if ($rootScope.currentUser) {
      return $rootScope.currentUser.currency;
    } else {
      return 'EUR';
    }
  }


  // Own back button ("guardar"):

  $scope.goBack = function() {
    $ionicHistory.goBack();
  };

  $scope.hideTabs = function () {
    // works fine but is called many many times while doing something in the app, so potentially slow?
    // console.log("state", $state.current.name);
    switch ($state.current.name) {
      case 'tab.createjob.details.photos':
        return true;
      case 'tab.createjob.details.description':
        return true;
      case 'tab.createjob.details.price':
        return true;
      case 'tab.createjob.details.date':
        return true;
      case 'tab.createjob.details.cleaning':
        return true;
      case 'tab.createjob.details.location':
        return true;

      case 'tab.searchjob.showjob':
        return true;
      case 'tab.searchjob.showjob.showlocation':
        return true;

      case 'tab.directory.search':
        return true;

      case 'tab.local.jobberdetails':
        return true;
      case 'tab.local.jobberdetails.showlocation':
        return true;
      case 'tab.local.showjob':
        return true;
      case 'tab.local.showjob.showlocation':
        return true;

      case 'tab.myjobs.selectjobber':
        return true;
      case 'tab.myjobs.selectjobber.showlocation':
        return true;
      case 'tab.myjobs.confirmedjob':
        return true;
      case 'tab.myjobs.confirmedjob.showlocation':
        return true;
      case 'tab.myjobs.applieddetails':
        return true;
      case 'tab.myjobs.applieddetails.showlocation':
        return true;
      case 'tab.myjobs.selectjobber.edit.photos':
        return true;
      case 'tab.myjobs.selectjobber.edit.description':
        return true;
      case 'tab.myjobs.selectjobber.edit.price':
        return true;
      case 'tab.myjobs.selectjobber.edit.date':
        return true;
      case 'tab.myjobs.selectjobber.edit.location':
        return true;
      case 'tab.myjobs.location':
        return true;
      case 'tab.myjobs.jobberdetails':
        return true;

      case 'tab.account.jobberpage.service':
        return true;
      case 'tab.account.settings':
        return true;
      case 'tab.account.showlocation':
        return true;

      default:
        return false;
    }
  }

  $scope.doneTick = function () {
    // works fine but is called many many times while doing something in the app, so potentially slow?
    // console.log("state", $state.current.name);
    switch ($state.current.name) {
      case 'tab.createjob.details.photos':
        return true;
      case 'tab.createjob.details.description':
        return true;
      case 'tab.createjob.details.price':
        return true;
      case 'tab.createjob.details.date':
        return true;
      case 'tab.createjob.details.cleaning':
        return true;

      case 'tab.directory.search':
        return true;

      case 'tab.myjobs.selectjobber.edit.photos':
        return true;
      case 'tab.myjobs.selectjobber.edit.description':
        return true;
      case 'tab.myjobs.selectjobber.edit.price':
        return true;
      case 'tab.myjobs.selectjobber.edit.date':
        return true;
      case 'tab.account.settings':
        return true;

      case 'tab.account.jobberpage.basic':
        return true;
      case 'tab.account.jobberpage.contact':
        return true;
      case 'tab.account.jobberpage.insurance':
        return true;
      case 'tab.account.jobberpage.business':
        return true;
      case 'tab.account.jobberpage.service.category':
        return true;
      case 'tab.account.jobberpage.service.description':
        return true;
      case 'tab.account.jobberpage.service.photos':
        return true;
      case 'tab.account.jobberpage.service.price':
        return true;

      default:
        return false;
    }
  }

  $scope.scrollTop = function (handle){
    $ionicScrollDelegate.$getByHandle(handle).scrollTop();
  }

  $scope.hipmob = function () {
    if ($rootScope.currentUser) {
      if(!$rootScope.currentUser.openedHipmobBefore && localStorage.getItem('uuid') != null) {
        $rootScope.currentUser.openedHipmobBefore = true;
        hipmob.set_name($rootScope.currentUser.name);
        hipmob.set_email($rootScope.currentUser.email);
        hipmob.set_phone($rootScope.currentUser.phone);
      }
    }
    hipmob.open();
    };



  var introSeen = window.localStorage.getItem("introSeen");
  var language  = window.localStorage.getItem("language");
  $timeout(
    function () {
      if (!introSeen || !language) {
        $rootScope.showIntro();
      }
    },500);

});
