angular.module('starter.controllers')

.controller('JobberpageCtrl', function($scope, $rootScope, $timeout, $ionicPopup, $state, $ionicLoading, JobberPage, jobCategories, gettextCatalog, User, JobMap, ValidateDNI, me) {

  $scope.state = $state.current.name;
  $scope.businessMap = new JobMap();
  $scope.canvas = $state.current.data.map_canvas;
  $scope.business_position = [];
  $scope.jobCategories = jobCategories;

  // Controller gets initilized. So the rosolved "me" needs to be set as currentUser.
  $scope.setCurrentUser(me);
  $rootScope.refreshCurrentUser = false;

  $scope.doRefreshCurrentUser = function() {
    $ionicLoading.show();

    User.get().$promise.then(function(me) {
      $scope.setCurrentUser(me);
      $rootScope.refreshCurrentUser = false;
    })
    .finally(function() {
     // Stop the ion-refresher from spinning
     $ionicLoading.hide();
     $scope.$broadcast('scroll.refreshComplete');
    })
  }

  $scope.refreshMap = function() {
    if ($rootScope.currentUser && $rootScope.currentUser.jobber_page && $rootScope.currentUser.jobber_page.fixed_position) {
      $scope.business_position = $rootScope.currentUser.jobber_page.fixed_position;
      $scope.fix = true;
    } else if ($rootScope.currentUser && $rootScope.currentUser.position) {
      $scope.business_position = $rootScope.currentUser.position;
    } else {
      $scope.business_position = [39.5735, 2.6492];
    }     
    $scope.businessMap.initMap($scope, $state.current.data.map_canvas, $scope.business_position, true);
    //if (!$scope.fix) $scope.businessMap.AddCurrentLocationPoint($scope, false, false, true);
    if ($scope.fix) {
      $scope.businessMap.addMarkersToMap($scope.business_position, ["Business"], "img/map_pin.png"); 
    }
    
    $scope.businessMap.SetCenter($scope.business_position);  
  }

  $scope.$on("$ionicView.enter", function() {
    if ($rootScope.refreshCurrentUser) {
      $scope.doRefreshCurrentUser();
    }

    if ($rootScope.refreshBusinessMap) {
      $rootScope.refreshBusinessMap = false;
    }
  });

  // AUTO-SAVE FUNCTION 
  // http://adamalbrecht.com/2013/10/30/auto-save-your-model-in-angular-js-with-watch-and-debounce/

  var timeout = null;
  var saveInProgress = false;

  var saveFinished = function() { saveInProgress = false; };

  var updateJobberPage = function() {
    if (saveInProgress==false && $rootScope.currentUser) {
      saveInProgress = true;
      var jobberPageData = {
        // id: $rootScope.currentUser.jobber_page ? $rootScope.currentUser.jobber_page.id : null,
        jobber_page: {
          id: $rootScope.currentUser.jobber_page ? $rootScope.currentUser.jobber_page.id : null,
          user_id: $rootScope.currentUser.id,
          business_name: $rootScope.currentUser.jobber_page.business_name,
          username: $rootScope.currentUser.jobber_page.username,
          fixed_position: $rootScope.currentUser.jobber_page.fixed_position,
          address: $rootScope.currentUser.jobber_page.address,
          address_details: $rootScope.currentUser.jobber_page.address_details
        }
      };
      JobberPage.update(jobberPageData).$promise.finally(saveFinished);
    }
  }

  var updateUser = function() {
    if (saveInProgress==false && $rootScope.currentUser) {
      saveInProgress = true;
      var userData = {
        name:             $rootScope.currentUser.name, 
        description:      $rootScope.currentUser.description,
        phone:            $rootScope.currentUser.phone,
        public:  $rootScope.currentUser.public,
      };
      User.update(userData).$promise.finally(saveFinished);

    }
  }

  var debounceSaveUpdatesUser = function(newVal, oldVal) {
    if (newVal != oldVal) {
      if (timeout) {
        $timeout.cancel(timeout)
      }
      timeout = $timeout(updateUser, 500);  // 500 = 0.5 seconds
    }
  };

  $scope.$watch('currentUser.name', debounceSaveUpdatesUser);
  $scope.$watch('currentUser.description', debounceSaveUpdatesUser);
  $scope.$watch('currentUser.phone', debounceSaveUpdatesUser);
  $scope.$watch('currentUser.public', debounceSaveUpdatesUser);

  

  var debounceSaveUpdatesJobberPage = function(newVal, oldVal) {
    if (newVal != oldVal) {
      if (timeout) {
        $timeout.cancel(timeout)
      }
      timeout = $timeout(updateJobberPage, 500);  // 500 = 0.5 seconds
    }
  };

  $scope.$watch('currentUser.jobber_page.business_name', debounceSaveUpdatesJobberPage);
  $scope.$watch('currentUser.jobber_page.username', debounceSaveUpdatesJobberPage);
  $scope.$watch('currentUser.jobber_page.address', debounceSaveUpdatesJobberPage);
  $scope.$watch('currentUser.jobber_page.address_details', debounceSaveUpdatesJobberPage);
  $scope.$watch('currentUser.jobber_page.fixed_position', debounceSaveUpdatesJobberPage);
  $scope.$watch('currentUser.public', debounceSaveUpdatesUser);


  $scope.changeEmail = function() {
    $scope.data = {};
    $scope.data.email = $rootScope.currentUser.email;
    // An elaborate, custom popup
    var myPopup = $ionicPopup.show({
      template: gettextCatalog.getString('Email') + ':* <input autofocus type="email" ng-model=data.email>',
      title: gettextCatalog.getString('Cambiar email'),
      scope: $scope,
      buttons: [
        { text: gettextCatalog.getString('Cancelar') },
        {
          text: gettextCatalog.getString('Cambiar'),
          type: 'button-positive',
          onTap: function(e) {
            if (!$scope.data.email)  {
              //don't allow the user to close unless he enters a star rating
              e.preventDefault("Por favor pon todos los datos.");
            } else {
              return $scope.data.email;
            }
          }
        },
      ]
    });
    myPopup.then(function(res) {
      if (res) {
        $rootScope.currentUser.email = res;
        User.update({user: {email: res}})
      }
    });
  }

  $scope.changeDNI = function() {
    // An elaborate, custom popup
    $scope.data = {};
    $scope.data.dni = $rootScope.currentUser.dni;
     // $rootScope.currentUser.dni;
    var myPopup = $ionicPopup.show({
      template: gettextCatalog.getString('DNI/NIE') + ': * <input autofocus type="text" ng-model=data.dni><br>' + gettextCatalog.getString('Recuerda darnos tu nombre correcto para que sea valido tu seguro.'),
      title: gettextCatalog.getString('DNI/NIE'),
      subTitle: gettextCatalog.getString('Si eres jobber da nos tu DNI/NIE para cubrirte automaticamente con un seguro de responsbilidad civil de nuestro partner AXA.') + '<br>' + gettextCatalog.getString('(Los contratantes podrán ver que tienes cubertura de seguro RC.)'),
      scope: $scope,
      buttons: [
        { text: gettextCatalog.getString('Cancelar') },
        {
          text: gettextCatalog.getString('Guardar'),
          type: 'button-positive',
          onTap: function(e) {
            if (!$scope.data.dni)  {
              // delte DNI if empty
              return ("delete_dni");
            } else if (!ValidateDNI.isDNI($scope.data.dni)){
              e.preventDefault();
              alert(gettextCatalog.getString("Not a correct DNI/NIE"));
            }
            else {
              return $scope.data.dni;
            }
          }
        }
      ]
    });
    myPopup.then(function(res) {
      if (res) {
        if (res == "delete_dni") res = "";
        $rootScope.currentUser.dni = res.toUpperCase();
        User.update({user: {dni: res.toUpperCase()}})
      }
    });
  }
  $scope.goToChildState = function(childState, params) {
    if ($rootScope.currentUser.public) {
      $state.go(childState, params);
    } else {
      var confirmPopup = $ionicPopup.confirm({
        title: gettextCatalog.getString('¡Ojo! Tu perfil aun no esta público.'),
        template: 'Solo puedes acceder a estos campos cuando tu perfil este público. ¿Quieres publicar tu perfil ahora?',
        buttons: [
          { text: gettextCatalog.getString('Cancelar') },
          { 
            text: gettextCatalog.getString('Sí, publicar'),
            type: 'button-positive',
              onTap: function(e) {
                return true;
              }
          }
        ]
      });
      confirmPopup.then(function(answer) {
        if(answer) {
          $rootScope.currentUser.public = true;
          $state.go(childState, params);
        }
      });
    }  
  }

})


.controller('JobberpageLocationCtrl', function($scope, $rootScope, $state, $compile, $ionicHistory, JobMap, JobberPage) {
  $scope.state              = $state.current.name;
  $scope.businessMapDetail  = new JobMap();
  $scope.canvas             = $state.current.data.map_canvas;
  var updateJobberPage = function() {
    if ($rootScope.currentUser) {
      
      var jobberPageData = {
        // id: $rootScope.currentUser.jobber_page ? $rootScope.currentUser.jobber_page.id : null,
        jobber_page: {
          id:               $rootScope.currentUser.jobber_page ? $rootScope.currentUser.jobber_page.id : null,
          user_id:          $rootScope.currentUser.id,
          business_name:    $rootScope.currentUser.jobber_page.business_name,
          username:         $rootScope.currentUser.jobber_page.username,
          fixed_position:   $rootScope.currentUser.jobber_page.fixed_position,
          address:          $rootScope.currentUser.jobber_page.address,
          address_details:  $rootScope.currentUser.jobber_page.address_details
        }
      };
      JobberPage.update(jobberPageData).$promise.finally(
        function() {
          $scope.goBack();
        }
      );
    }
  }
  
  $scope.business_position = [];
  
  // TODO: Rename this variable something more expressive like has_fixed_position.
  $scope.fix = false;

  if ($rootScope.currentUser && 
      $rootScope.currentUser.jobber_page && 
      $rootScope.currentUser.jobber_page.fixed_position ) {
    $scope.business_position  = $rootScope.currentUser.jobber_page.fixed_position;
    $scope.fix                = true;
  } else if ($rootScope.currentUser && $rootScope.currentUser.position) {
    $scope.business_position  = $rootScope.currentUser.position;
  } else {
    $scope.business_position  = [39.5735, 2.6492];
  } 
  
  $scope.businessMapDetail.initMap(
    $scope,
    $state.current.data.map_canvas,
    $scope.business_position );
  
  if (!$scope.fix) {
    // We do not have a fixed_position yet. Take current position.
    $scope.businessMapDetail.AddCurrentLocationPoint(
      $scope,
      true,
      false,
      true );
  } else {
    // We are editing an existing fixed_position.
    $scope.address          = $rootScope.currentUser.jobber_page.address;
    $scope.latLng           = $rootScope.currentUser.jobber_page.fixed_position;
    $scope.address_details  = $rootScope.currentUser.jobber_page.address_details;
    var contentString       = "<div location-info></div>";
    var compiled            = $compile(contentString)($scope);
    
    $scope.businessMapDetail.addMarkersToMap(
      $scope.business_position, 
      compiled,
      "img/map_pin.png",
      true ); 
  }

  $scope.businessMapDetail.SetCenter($scope.business_position);  
  
  $scope.searchBox = $scope.businessMapDetail.linkSearchBox($scope, "map_searchbox");

  $scope.goToCurrent = function() {
    angular.element(document.getElementById('map_searchbox').blur());
    $scope.businessMapDetail.clearMarkers();
    $scope.businessMapDetail.AddCurrentLocationPoint($scope, true);
    $scope.searchLocation = "";
  }

  $scope.getAddress = function(latLng) {
    $scope.businessMapDetail.getAddress(latLng).then(function (res) {
      //$scope.searchLocation=res;
    })
  }

  $scope.clearSearch = function(latLng) {
    $scope.searchLocation = "";
  }
  
  $scope.selectLocation = function(latLng, adr, address_details) {
    $rootScope.currentUser.jobber_page.fixed_position   = latLng;
    $rootScope.currentUser.jobber_page.address          = adr;
    $rootScope.currentUser.jobber_page.address_details  = address_details;
  
    $rootScope.refreshBusinessMap = true;

    if ($ionicHistory.viewHistory().stateName  == 'tab.account.jobberpage.location'){
      $scope.goBack();  
    } else {
      updateJobberPage();
    }    
  }
	
  $scope.editAddress = function(adr) {
    $scope.businessMapDetail.clearMarkers();
    $scope.searchLocation = adr;
    document.getElementById('map_searchbox').focus();
  }

})

;
