angular.module('starter.controllers')

// ============================ Search Job Tab ============================
// For Search Job Tab and all sub pages

.controller('SearchJobCtrl', function($scope, $compile, $state, Job, $rootScope, $ionicModal, GeoLocation, $timeout, gettextCatalog, jobCategories) {

  // When jobs are loaded from DB:
  $scope.state = $state.current.name;
  $scope.doRefresh = function() {

    $scope.jobs = Job.query();
    $scope.jobs.$promise.then(function(res) {
      // For list view: update distances on current location changed event -
      // only set event listener once, not for every tab change
      if (!$rootScope.SearchJobCtrl_current_location_changed) {
        console.log("set location listener in SearchJobCtrl");
        $rootScope.SearchJobCtrl_current_location_changed = $rootScope.$on('current_location_changed', function(eventx, data) {
          console.log("current: ", GeoLocation.currentLocation)
          $scope.locationListenerSet = true;
          console.log("update distances");
          $scope.jobs = _.map($scope.jobs, function(job) {
            if (job.position) {
              job.distance = GeoLocation.distanceToCurrent(job.position);
            } else {
              job.distance = '';
            }
            return job;
          });
        });
      }
	  })
    .finally(function() {
     // Stop the ion-refresher from spinning
     $scope.$broadcast('scroll.refreshComplete');
    });

    $rootScope.refreshSearchJobs = false;
  }

  $scope.$on("$ionicView.beforeEnter", function() {
    if($rootScope.refreshSearchJobs) {
      $scope.doRefresh();
    }
  });

  // Build map:

  $scope.doRefresh();

  $scope.$on('refreshMap', function() {
    console.log("refresh map listener");
    $timeout(function() {
      $scope.refreshMap();
    }, 1);
  });

    //Slider Box
  $scope.showSliderBox = function(templateUrl) {
    $ionicModal.fromTemplateUrl(templateUrl, {
      scope: $scope,
      animation: 'slide-in-up'
    }).then(function(modal) {
      $scope.modal = modal;
      $scope.modal.show();
    });
  }
  $scope.showImages = function(index) {
    $scope.activeSlide = index;
    $scope.showSliderBox('templates/_job-photo-slider.html');
  }
  $scope.closeModal = function() {
    $scope.modal.hide();
    $scope.modal.remove()
  };
})

.controller('ShowJobCtrl', function(job, $scope, $stateParams, $state, $filter, $ionicLoading, $ionicHistory, $ionicSlideBoxDelegate, $ionicModal, $ionicScrollDelegate, Job, JobMap, jobCategories, GeoLocation, $ionicPopup, User, Sharing, $ionicTabsDelegate, $rootScope, gettextCatalog) {

  console.log($stateParams);
  console.log("Job selected-->",job);
  $scope.call = $stateParams.call;
  $scope.categories = jobCategories;
  $scope.moment  =  window.moment;
  $scope.viewScope = function(){
    return $scope;
  };

  $scope.freshController = true;
  $scope.$on("$ionicView.beforeEnter", function() {
    if(!$scope.freshController) {
      Job.get({id: $stateParams.id})
        .$promise
        .then(function(job){
          $scope.job = job;
          $scope.$broadcast('job.refreshed');
        })
    }
    $scope.freshController = false;
  });

  $scope.$on('$ionicView.beforeLeave', function() {
    $scope.call = "";
  })

  $scope.doRefresh = function(){
    Job.get({id: $stateParams.id})
      .$promise
      .then(function(job){
        $scope.job = job;
        $scope.$broadcast('job.refreshed');
      })
  }

  var shareObject = {};
  $scope.state = $state.current.name;
	$scope.job = job;

  $rootScope.$on('current_location_changed', function(eventx, data) {
    console.log("current: ", GeoLocation.currentLocation)
    $scope.locationListenerSet = true;
    console.log("update distances");
    if ($scope.job.position) {
      $scope.job.distance = GeoLocation.distanceToCurrent($scope.job.position);
    }
  });


  $ionicSlideBoxDelegate.update();
  // prepare social sharing object for this job:
  shareObject.shareMessage = job.jobCategory + ": " + $filter('jmPriceCurrency')(job.price, job.currency) + ", " + job.description;
  shareObject.shareSubject = gettextCatalog.getString("Te interesa este job?");
  if (job.photo_file_names.length == 0) {
    shareObject.shareImage = jobCategories.get(job.category_id).fbImage;
  } else {
    shareObject.shareImage = job.imagePath;
  }
  shareObject.shareLink = "https://jobmapp.com/jobs/" + job.id;

  $scope.activate = function(job) {
    if (confirm('Acitvate this job and make it visible to everybody?')) {
      Job.activate({id: job.id}).$promise.then(function(res){
        // activating job was successful.
        $rootScope.refreshSearchJobs = true;
        $scope.doRefresh();
      }, function(res){
        // activating job was not successful.
        alert("Error while activating the job :-(\n We will reload the job now.");
        $scope.doRefresh();
      })
    }
  }

  $scope.deny = function(job) {
    if (confirm('¿Deny and cancelar this job?')) {
      Job.deny({id: job.id}).$promise.then(function(res){
        // activating job was successful.
        $rootScope.refreshSearchJobs = true;
        $scope.doRefresh();
      }, function(res){
        // activating job was not successful.
        alert("Error while denying/canceling the job :-(\n We will reload the job now.");
        $scope.doRefresh();
      })
    }
  }

  $scope.compartirJob = function(channel) {
    $ionicLoading.show({templateUrl : 'templates/loading-animation.html', hideOnStateChange: true, duration: 10000});
    var urlParams = {
      job_id : job.id,
      $desktop_url : shareObject.shareLink,
      $android_url : shareObject.shareLink,
      $ios_url : shareObject.shareLink
    };
    if ($rootScope.currentUser) {
      urlParams.referringUser = {
        uuid: $rootScope.currentUser.uuid ,
        name: $rootScope.currentUser.name,
        picture: $rootScope.currentUser.bigImagePath()
      }
    }
    console.log("Share in ", channel);
    console.log("This urlParams", urlParams);
//     BranchIo.getShortUrl(urlParams, "app", "jobsharing").then(function(url) {
//       console.log("url", url);
//       shareObject.shareLink = url;
//       if(channel == 'facebook'){
//         Sharing.shareFacebook(shareObject);
//       }
//       if(channel == 'sharing'){
//         Sharing.shareOthers(shareObject);
//       }
//       if(channel == 'twitter'){
// //        Sharing.shareOthers(shareObject);
//       }
//       if(channel == 'whatsapp'){
//         Sharing.shareWhatsapp(shareObject);
//       }
// //      Sharing.shareAction(shareObject, gettextCatalog.getString("Compartir con..."));
//     });
  }

  $scope.showGeneralConditions = function() {
    var myPopup2 = $ionicPopup.alert({
      templateUrl: 'templates/_general-conditions.html',
      title: gettextCatalog.getString('Condiciones generales')
    });
  }

  var showContactCheckPopup = function() {
    $scope.job.applicationComment = null;

    var myPopup = $ionicPopup.show({
      template: "<div class='form contact'>" +gettextCatalog.getString('Email') + ':* <input ng-model=currentUser.email><br>' + gettextCatalog.getString('Teléfono') + ':* <input ng-model=currentUser.phone><br>' + gettextCatalog.getString('Explica aquí porque deben elegirte') + ': <textarea rows="4" ng-model="job.applicationComment" placeholder="'+ gettextCatalog.getString('Creo que debes elegirme ya que...') +'" class="explain"></textarea><br><label><input type="checkbox" ng-model="currentUser.terms_accepted" class="big-checkbox">' + gettextCatalog.getString('* Acepto las') + ' </label><a href ng-click="showGeneralConditions()">' + gettextCatalog.getString('condiciones generales') + '</a></div>',
      title: "<h2 class='title contact'>" + gettextCatalog.getString('Datos de contacto') + "</h2>",
      subTitle: '<p class="subtitle contact">' + gettextCatalog.getString('Comprueba que tus datos de contacto son correctos') + '</p>',
      scope: $scope,
      buttons: [
        { text: gettextCatalog.getString('Cancelar') },
        {
          text: gettextCatalog.getString('OK'),
          type: 'button-positive',
          onTap: function(e) {
            if (!$rootScope.currentUser.email || !$rootScope.currentUser.phone || !$rootScope.currentUser.terms_accepted) {
              //don't allow the user to close unless he enters a star rating
              e.preventDefault();
            } else {
              return $rootScope.currentUser;
            }
          }
        },
      ]
    });
    myPopup.then(function(res) {
      if (res) {
        User.update({user: {email: $rootScope.currentUser.email, phone: $rootScope.currentUser.phone, terms_accepted: true}});
        $ionicLoading.show({template: gettextCatalog.getString("Mandando solicitud"), hideOnStateChange: true, duration: 10000});
        console.log("////////////////", {job: {id: $stateParams.id, application_comment: $scope.applicationComment}});
  			Job.apply({job: {id: $stateParams.id, application_comment: $scope.job.applicationComment}}).$promise
          .then(
            function () {
              $ionicLoading.hide();
              $rootScope.refreshSearchJobs = true;
              var alertPopup = $ionicPopup.alert({
                title: gettextCatalog.getString('Solicitud enviada'),
                template: gettextCatalog.getString('Te informaremos si te asignan este trabajo.')
              });
              $ionicHistory.goBack();
              $state.go('tab.myjobs.applieddetails', {id: $stateParams.id});
            } ,
            function (error) {
              $ionicLoading.hide();
              var alertPopup = $ionicPopup.alert({
                title: gettextCatalog.getString('Error'),
                template: gettextCatalog.getString('Solicitud no ha sido posible. El trabajo ya no está disponible. - ') + error.status + ' ' + error.statusText,
              });
            }
          );
      } else {console.log("cancel popoup")};
    });
  } // end popoup

  $scope.jobApply = function() {
    // if not logged in open login popup, otherwise proceed with contact check popup and application
    if ($rootScope.currentUser == null) {
      $scope.showLoginPopup();
    } else {
      showContactCheckPopup();
		}
  }

  $scope.changeNotifications = function () {
    console.log("in change Notifications");
    $ionicTabsDelegate.select(3);
    console.log("state go");
    $state.go('tab.account.settings', {call: 'pushMsg'});
  }




  //Slider Box
  $scope.showSliderBox = function(templateUrl) {
 $ionicModal.fromTemplateUrl(templateUrl, {
 scope: $scope,
 animation: 'slide-in-up'
 }).then(function(modal) {
 $scope.modal = modal;
 $scope.modal.show();
 });
 }
  $scope.showImages = function(index) {
 $scope.activeSlide = index;
 $scope.showSliderBox('templates/_job-photo-slider.html');
 }
  $scope.closeModal = function() {
 $scope.modal.hide();
 $scope.modal.remove();
 };





})

;
