angular.module('mlz.backbutton', ['ionic'])
  .controller('BackButtonCtrl', function($scope, $log, $state, $ionicHistory, $ionicViewSwitcher, $rootScope) {
    
    $rootScope.$on('$ionicView.beforeEnter', function(e, data){
      $rootScope.isBackButtonShown = data.enableBack;
    });


    var ctrl = this;
    $scope.fakeState = {
      show: true,
      previousState: ''
    };
    
    /**
     * Checks to see if there is a view to navigate back to
     * if we have no current back view.
     *
     * @param  {string} state The current view state
     * @return {boolean}      If there is a view we should navigate back to
     */
    
    this.showNonNativeBackBtn = function (stateName) {

      if (!stateName) {
        // No stateName provided. Set stateName to the current state we are in.
        stateName = $ionicHistory.currentStateName();
      }
      var statePath = stateName.split('.');
      
      // This block will decide whether we need our own back button or not.
      if (statePath.length > 2) {
        // The parent path is the statePath without the last element.
        var parentPath = _.take(statePath, statePath.length - 1);
        var parentStateName = parentPath.join('.');
        
        if ($rootScope.isBackButtonShown){
          // The parent state was already visited, so we do not need our own back button.
          console.log("NO need for back button DEEP");
          return false;
        } else {
          // The parent state was not yet visited. We need our additional back button.
          console.log("need back button");
          // This is where the back button links to
          ctrl.toState = parentStateName;
          return true;
        }
      } else {
        // We are in main view of a tab.
        console.log("NO need for back button HIGH");
        return false;
      }
    };

    $scope.mlzGoBack = function () {

      // check we can go back first
      if (!ctrl.showNonNativeBackBtn()) {
        return false;
      }

      // Switch this animation around so it looks like we're navigating backwards
      $ionicViewSwitcher.nextDirection('back');

      // Stop it from caching this view as one to return to
      $ionicHistory.nextViewOptions({
        historyRoot: true,
        disableBack: true
      });

      // Switch to our new previous state
      $state.go(ctrl.toState, $state.params);
    };

    $scope.fakeState.show = ctrl.showNonNativeBackBtn();

    $scope.$on('$stateChangeStart', function (e, to) {
      $scope.fakeState.show = ctrl.showNonNativeBackBtn(to.name);
    });

  })


  .directive('mlzBackButton', function() {
    return {
      restrict: 'E',
      replace: true,
      controller: 'BackButtonCtrl',
      template: [
        '<ion-nav-buttons> ',
          '<button class="button back-button mlz-back-button buttons button-clear header-item" ng-if="!isBackButtonShown" ng-click="mlzGoBack()">',
            '<i class="icon ion-ios-arrow-left"></i>',
          '</button>',
        '</ion-nav-buttons>'
      ].join('')
    };
  })
;