// Branch IO docs found here:
// https://github.com/BranchMetrics/Branch-PhoneGap-Cordova-SDK
angular.module('jm.branchIo', ['ionic'])
.factory('BranchIo', function ($q, $ionicPlatform, $rootScope, env) {

    var deepLink = {};
    var referringUser = {};
    var onDevice = false;
    var initialized = false;
    var campaign_code = "";
    var job_id = null;
    var jobber_id = null;

    var getLatestFirstParams = function () {
      var deferred = $q.defer();
      var branch = window.Branch;
      if (_.isEmpty(deepLink) || true) {
        branch.getLatestReferringParams(function(data){
          if (data) {
            console.log("1) use latest referring params:", data);
            deepLink = data;
            referringUser = data.referringUser;
            data.step = "latest";
            deferred.resolve(data);
          } else {
            branch.getFirstReferringParams(function(data){
              if (data) {
                console.log("2) use first referring params:", data);
                deepLink = data;
                referringUser = data.referringUser;
                data.step = "first";
                deferred.resolve(data);
              } else {
                console.log("3) cannot find any first referring params");
                deferred.resolve();
              }
            });
          }
        });
      } else {
        console.log("0) deepLink already set");
        deferred.resolve(deepLink);                    
      }  
      return deferred.promise;
    };


    return {
        
        init: function() {

          var branch = window.Branch;
          var deferred = $q.defer();
          branch.initSession(true, function(params) {
              console.log("init session call back");
              console.log('Returned params: ' + JSON.stringify(params));
              initialized = true;
              deepLink = params;
              if (params.referringUser) {             
                referringUser = params.referringUser;
              }
              if (params.campaign_code) {
                campaign_code = params.campaign_code;
                console.log("Campaign: ", campaign_code);
              } 
              if (params.job_id) {
                job_id = params.job_id;
              }
              if (params.jobber_id) {
                jobber_id = params.jobber_id;
              }
              $rootScope.$broadcast('branch_io_initialized', params);
              deferred.resolve(params);
          });

          return deferred.promise;

        },

        initParams: function() {
          result = {};
          console.log("in initParams");
          if (job_id || jobber_id || !_.isEmpty(referringUser)) {
            result.initSuccess = true;
            result.job_id = job_id;
            result.jobber_id = jobber_id;
            result.referringUser = referringUser;
            console.log("init Params sucesss", result);
          } else {
            result.initSuccess = false;
          }
          return result;
        },

        pause: function() {

          console.log("resetting BranchIo");
          deepLink = {};
          referringUser = {};
          campaign_code = "";
          job_id = null;  
          jobber_id = null;        
          
          console.log("Pause Session");
          var branch = window.Branch;
          branch.closeSession();  

        }, 


        setIdentity: function(identity) {

          var branch = window.Branch;

          if (window.cordova) {
            if (initialized == true) {
              branch.setIdentity(identity, null);
              console.log("set identity: identity set", identity);
            } else {
              var deleteListener = $rootScope.$on('branch_io_initialized', function (e) {
                console.log("set identity: listener received", identity);
                var branch = window.Branch;
                branch.setIdentity(identity, null); 
                deleteListener();                              
              });
            }
          }
            
        },

        logout: function() {

          var branch = window.Branch;

          if (window.cordova) {
            // branch.logout();
          }

        },

        getReferringParams: function() {

            var branch = window.Branch;
            var deferred = $q.defer();

            if (window.cordova) {
              if (initialized) {
                deferred.resolve(getLatestFirstParams());
              } else {
                console.log("BranchIo NOT initialized");
                var deleteListener = $rootScope.$on('branch_io_initialized', function (e) {
                  console.log("BranchIo initialized: now getReferringParams", referringUser, _.isEmpty(referringUser));
                  deferred.resolve(getLatestFirstParams());
                  deleteListener();
                  console.log("BranchIo: getReferringParams listener deleted");
                })
              }
            }

            return deferred.promise;

        },

        getRewardBalance: function(bucket) {

            var deferred = $q.defer();

            if (window.cordova) {
              var branch = window.Branch;
              branch.loadRewards(function(changed) {          
                if (changed) {
                  console.log("credits changed", changed);
                  // Pass the bucket name to retrieve the current balance of credits
                }
                branch.getCredits(bucket, function(credits) {
                  deferred.resolve(credits);
                });
              });
            }
            return deferred.promise;

        },

        getShortUrl: function(params, channel, feature) {

            var deferred = $q.defer();

            // if (window.cordova) {
            //   var branch = window.Branch;

            //   console.log("get url 2");
            //   branch.getShortUrl(params || {}, channel, feature, function(url) {
            //     console.log("get url 3", url);
            //     deferred.resolve(url);
            //   });
            // }

            return deferred.promise;

        },


        userCompletedAction: function(action) {

            // if (window.cordova) {
            //   var branch = window.Branch;

            //   branch.userCompletedAction(action, null);
            // }
            
        }

    };

})
.run(function($ionicPlatform, BranchIo, env, $state) {

  $ionicPlatform.ready(function() {
//     var debugMode = true;
//     if (env.name == 'production') debugMode = false;
//     if (debugMode) appKey = '129496897248317578'
//     else appKey = '129496897239928969'
    
//     var branch = window.Branch;
//     if (window.cordova) branch.getInstance(appKey, function() {
//       if (debugMode) {
//         console.log("set debug mode");
//         branch.setDebug();
//       }
//       document.addEventListener("pause", BranchIo.pause, false);
//       console.log("call init from run"); 
//       BranchIo.init().then(function(params) {  
// //        alert("run done:" + JSON.stringify(params)); 
//         if (params) {             
//           console.log("params: ", params);
//         }
//       });    
//     });

//     // Disable BACK button on home
//       $ionicPlatform.registerBackButtonAction(function (event) {
//         console.log("close the app", $state.current);
//         if($state.current.name == "tab.searchjob" || $state.current.name == "tab.myjobs" || $state.current.name == "tab.createjob" || $state.current.name ==  "tab.account") {
//           BranchIo.pause();
//           console.log("Branch paused");
//           navigator.app.exitApp();
//         }
//         else {
//           navigator.app.backHistory();
//         }
//       }, 100);

  });
});