angular.module('starter.services')
.factory('popupPayment', function($ionicPopup, $q, $ionicLoading, $ionicHistory, $timeout, gettextCatalog, Braintree, env, jobForm){

  return {

    getPaymentInfo: function (scope, job) {
      var q = $q.defer();

      scope.ccdata = {};
      scope.ccdata.jobId = job.id;
      //scope.ccdata.jobId = 159;
      scope.ccdata.braintreeClientToken = "";

      if (env.name != 'production') {
        scope.ccdata.number = "4111111111111111";
        scope.ccdata.expiration_month = "10";
        scope.ccdata.expiration_year = "20";
      }

      Braintree.getClientToken()
        .then(
          function (res) {
            scope.ccdata.braintreeClientToken = res.data.client_token;
          },
          function(err) {
            console.log("error obtaining braintree client token", err);
          }
        );

      var paymentPopup = $ionicPopup.confirm({
        templateUrl: 'templates/_payment.html',
        title: gettextCatalog.getString('Autorizar pago'),
        subTitle: gettextCatalog.getString('Únicamente se cobra en el momento de la entrega'),
        scope: scope,
        buttons: [
         { text: gettextCatalog.getString('Cancelar') },
         {
           text: gettextCatalog.getString('Contratar'),
           type: 'button-positive',
           onTap: function(e) {
            if (!scope.ccdata.number || !scope.ccdata.expiration_month || !scope.ccdata.expiration_year || !scope.ccdata.braintreeClientToken) {
              //don't allow the user to close OK unless he enters credit card info
              e.preventDefault();
            } else {
              return scope.ccdata;
            }
           }
         },
        ]
      });

      paymentPopup.then(function(res) {
        if (res) {
          // return credit card data:
          q.resolve(res);
        } else {
          console.log("popoup canceled");
          q.reject("User canceled payment screen");
        };
      });

      return q.promise;

    },

    makePayment: function (paymentData) {
      var q = $q.defer();

      $ionicLoading.show({template: gettextCatalog.getString("Procesando pago"), hideOnStateChange: true, duration: 10000});
      Braintree.checkout(paymentData.braintreeClientToken, paymentData.number, paymentData.expiration_month, paymentData.expiration_year, paymentData.jobId)
      .then(
        function (res) {
          // show "pago realizado" instead of "procesando checkout" for 1000ms then proceed with assign jobber
          $timeout(function() {
            $ionicLoading.show({template: gettextCatalog.getString("Pago realizado"), hideOnStateChange: true, duration: 10000});
          }, 1);
          $timeout(function() {
            //jobAssign(candidateId);
            console.log("payment ok:", res);
            q.resolve(res);
          }, 1000);
        },
        function () {
          $ionicLoading.hide();
          var alertPopup = $ionicPopup.alert({
            title: gettextCatalog.getString('Pago no ha sido realizado'),
            template: gettextCatalog.getString('Vuelve a intentarlo o usa otra tarjeta de crédito.')
          });
          q.reject("Payment not successfull");
        }
      )
      .finally(function () {
        console.log("running finally");
        $ionicLoading.hide();
      });

      return q.promise;
    },

    authorizeServiceBooking: function (paymentData) {
      var q = $q.defer();

      $ionicLoading.show({template: gettextCatalog.getString("Procesando autorización"), hideOnStateChange: true, duration: 10000});
      Braintree.authorizeServiceBooking(paymentData.braintreeClientToken, paymentData.number, paymentData.expiration_month, paymentData.expiration_year, paymentData.jobId)
      .then(
        function (res) {
          // show "pago realizado" instead of "procesando checkout" for 1000ms then proceed with assign jobber
          $timeout(function() {
            $ionicLoading.show({template: gettextCatalog.getString("Pago autorizado"), hideOnStateChange: true, duration: 10000});
          }, 1);
          $timeout(function() {
            //jobAssign(candidateId);
            console.log("payment ok:", res);
            jobForm.resetDraft();
            $ionicHistory.nextViewOptions({
              historyRoot: true,
              disableBack: true,
              disableAnimation: true
            });   
            $ionicHistory.goBack();
            q.resolve(res);
          }, 1000);
        },
        function () {
          $ionicLoading.hide();
          var alertPopup = $ionicPopup.alert({
            title: gettextCatalog.getString('Autorización no ha sido posible'),
            template: gettextCatalog.getString('Vuelve a intentarlo o usa otra tarjeta de crédito.')
          });
          q.reject("Payment not successfull");
        }
      )
      .finally(function () {
        console.log("running finally");
        $ionicLoading.hide();
      });

      return q.promise;
    }


  }
});
