angular.module('starter.services')
.factory('popupSaveUpdateJob', function($ionicPopup, $q, $ionicLoading, $timeout, $ionicHistory, gettextCatalog, Job, jobForm){

  return {

    saveJob: function ($scope, job) {
      var q = $q.defer();

      // this is a brand new job.
      if (job.specialSubCategory == "ecoscooter") {
        $ionicLoading.show({template: gettextCatalog.getString("Guardando job"), hideOnStateChange: true, duration: 15000});
      } else {
        $ionicLoading.show({template: gettextCatalog.getString("Publicando job"), hideOnStateChange: true, duration: 15000});
      }
      Job.save({job: {
        category_id: job.category_id,
        photo_file_names: job.photo_file_names,
        description: job.description,
        price: job.price,
        currency: job.currency,
        position: job.position,
        address: job.address,
        address_details: job.address_details,
        // ATTENTION: The following fields only for save not for update
        address_hint: job.address_hint,
        contact_name: job.contact_name,
        contact_phone: job.contact_phone,
        position_2: job.position_2,
        address_2: job.address_2,
        address_details_2: job.address_details_2,
        address_hint_2: job.address_hint_2,
        contact_name_2: job.contact_name_2,
        contact_phone_2: job.contact_phone_2,
        service_id: job.service_id,
        date: job.date,
        time: job.time,
        settled : true
      }}, function(response) {
        // Nice, creating the job was successfull.
        console.log("job save response", response);
        $ionicLoading.hide();
        // Ecoscooter job procede with payment:
        if (job.specialSubCategory == "ecoscooter") {
          q.resolve(response);
        // Standard job
        } else {
          $scope.enablePublishBtn = true;
          $ionicHistory.goBack();
          var alertPopup = $ionicPopup.show({
            title: gettextCatalog.getString('Job publicado'),
            template: '<p>' + gettextCatalog.getString('Te informaremos cuando alguien se interesa por el job. Puedes ver el estado en "Mis Jobs"') + '</p><p>' + gettextCatalog.getString('Quieres publicar el job en tu facebook también?') + '</p>',
            buttons: [
              { text: gettextCatalog.getString('Cerrar') },
              {
                text: '<b>' + gettextCatalog.getString('Facebook') + '</b>',
                type: 'button-positive',
                onTap: function(e) {
                  return true;
                }
              }
            ]
          });
          alertPopup.then(function(answer) {
            // Resetting job draft object for new jobs.
            jobForm.resetDraft();
            if (answer) {
              $ionicLoading.show({templateUrl: 'templates/loading-animation.html', hideOnStateChange: true, duration: 20000});
              var shareObject = {};
              // shareObject texts are not really used in facebook (og meta tags from the link are used) I put them here anyway:
              shareObject.shareMessage = job.description;
              shareObject.shareSubject = gettextCatalog.getString("Alguien me puede echar una mano?");
              if (job.photo_file_names.length > 0) {
                shareObject.shareImage = job.imagePath;
              }
              shareObject.shareLink = env.apiEndPoint + "/jobs/" + response.id;
              Sharing.shareFacebook(shareObject);
            }
            $ionicHistory.nextViewOptions({
              historyRoot: true,
              disableBack: true,
              disableAnimation: true
            });   
            $ionicHistory.goBack();
            q.resolve(response);       
          });
        }

      }, function(res){
        // Oops. Crating the job was not successfull. 
        console.log("error saving job", res);
        $ionicLoading.hide();
        var alertPopup = $ionicPopup.alert({
          title: gettextCatalog.getString('Error publicando el job'),
          template: gettextCatalog.getString('Intentalo de nuevo o ponte en contacto con nosotros')
        });
        $scope.enablePublishBtn = true;
        q.reject();
      });
  
      return q.promise;
    },
      
    updateJob: function ($scope, job) {
      var q = $q.defer();

      $ionicLoading.show({template: gettextCatalog.getString("Actualizando job"), hideOnStateChange: true, duration: 15000});
      Job.update({job: {
        id: job.id,
        category_id: job.category_id,
        photo_file_names: job.photo_file_names,
        description: job.description,
        price: job.price,
        currency: job.currency,
        position: job.position,
        address: job.address,
        address_details: job.address_details,
        service_id: job.service_id,
        date: job.date,
        time: job.time,
        settled : true
      }}, function(response) {
        // Update was successfull
        $scope.enablePublishBtn = true;
        $ionicLoading.hide();
      
        var alertPopup = $ionicPopup.show({
          title: gettextCatalog.getString('Job actualizado'),
          template: '<p>' + gettextCatalog.getString('Notificaremos de los cambios que has hecho a todos los que se interesaron hasta el momento. Si alguien más se interesa por el job, ¡te avisaremos! +INFO en "Mis Jobs".') + '</p>',
          buttons: [
            { text: gettextCatalog.getString('OK') }
          ]
        });
        alertPopup.then(function(res) {
          // Resetting job draft object for new jobs.
          jobForm.setToUpdate(false);
          $ionicHistory.nextViewOptions({
            historyRoot: true,
            disableBack: true,
            disableAnimation: true
          });
          $ionicHistory.goBack();
          q.resolve(response);   
          //$ionicHistory.clearHistory();
        });
      }, function(res){
        // Oops. Update was not happy.
        console.log("error updating job", res);
        $ionicLoading.hide();
        var alertPopup = $ionicPopup.alert({
          title: gettextCatalog.getString('Error actualizando el job'),
          template: gettextCatalog.getString('Intentalo de nuevo o ponte en contacto con nosotros')
        }); 
        $scope.enablePublishBtn = true;
        q.reject();
      });   

      return q.promise;
    }
  }
});